# encoding: UTF-8

from abc import abstractmethod
import abc
from six import with_metaclass
import copy
from collections import defaultdict

import numpy as np

from jaqs.data.basic import *
from jaqs.data.basic import Position
from jaqs.data.basic import Trade
from jaqs.util.sequence import SequenceGenerator
import jaqs.util as jutil
from jaqs.trade.event import EVENT_TYPE, EventEngine, Event
from jaqs.trade.tradeapi import TradeApi
from jaqs.data.basic import OrderRsp, OrderStatusInd, Trade, TaskInd, Task


'''
class TradeCallback(with_metaclass(abc.ABCMeta)):
    @abstractmethod
    def on_trade(self, ind):
        pass
    
    @abstractmethod
    def on_order_status(self, ind):
        pass
    
    @abstractmethod
    def on_order_rsp(self, rsp):
        pass

'''


class BaseTradeApi(object):
    def __init__(self):
        super(BaseTradeApi, self).__init__()
        
        self._order_status_callback = None
        self._task_status_callback = None
        self._trade_callback = None
        self._on_connection_callback = None
        
        self.ctx = None
        
    def set_connection_callback(self, callback):
        self._on_connection_callback = callback
    
    def set_order_status_callback(self, callback):
        self._order_status_callback = callback
    
    def set_trade_callback(self, callback):
        self._trade_callback = callback
    
    def set_task_status_callback(self, callback):
        self._task_status_callback = callback
    
    def place_order(self, symbol, action, price, size, algo="", algo_param={}, userdata=""):
        """
        return (result, message)
        if result is None, message contains error information
        """
        pass
    
    def place_batch_order(self, orders, algo="", algo_param={}, userdata=""):
        """
        orders format:
            [ {"symbol": "000001.SZ", "action": "Buy", "price": 10.0, "size" : 100}, ... ]
        return (result, message)
        if result is None, message contains error information
        """
        pass
    
    def cancel_order(self, task_id):
        """
        return (result, message)
        if result is None, message contains error information
        """
        pass
    
    def query_account(self, format=""):
        """
            return pd.dataframe
        """
        pass
    
    def query_position(self, mode = "all", symbols = "", format=""):
        """
            symbols: seperate by ","
            return pd.dataframe
        """
        pass
    
    def query_net_position(self, mode = "all", symbols = "", format=""):
        """
            symbols: seperate by ","
            return pd.dataframe
        """
        pass
    
    def query_task(self, task_id = -1, format=""):
        """
            task_id: -1 -- all
            return pd.dataframe
        """
        pass
    
    def query_order(self, task_id = -1, format=""):
        """
            task_id: -1 -- all
            return pd.dataframe
        """
        pass
    
    def query_trade(self, task_id = -1, format=""):
        """
            task_id: -1 -- all
            return pd.dataframe
        """
        pass
    
    def query_portfolio(self, format=""):
        """
            return pd.dataframe
        """
        pass
    
    def goal_portfolio(self, positions, algo="", algo_param={}, userdata=""):
        """
        positions format:
            [ {"symbol": "000001.SZ", "ref_price": 10.0, "size" : 100}, ...]
        return (result, message)
        if result is None, message contains error information
        """
        pass
    
    def basket_order(self, orders, algo="", algo_param={}, userdata=""):
        """
        orders format:
            [ {"symbol": "000001.SZ", "ref_price": 10.0, "inc_size" : 100}, ...]
        return (result, message)
        if result is None, message contains error information
        """
        pass
    
    def stop_portfolio(self):
        """
        return (result, message)
        if result is None, message contains error information
        """
        pass
    
    def query_universe(self, format=""):
        pass


'''
class RealTimeTradeApi_OLD(BaseTradeApi):
    def __init__(self):
        super(RealTimeTradeApi, self).__init__()
        
        self.seq_gen = SequenceGenerator()

    def _get_next_num(self, key):
        """used to generate id for orders and trades."""
        return str(np.int64(self.ctx.trade_date) * 10000 + self.seq_gen.get_next(key))
    
    def _get_next_task_no(self):
        return self._get_next_num('task_no')

    def put_to_gateway(self, event):
        self.put_to_gateway(event)

    # ----------------------------------------------------------------------------------------
    # place & cancel
    
    def place_order(self, symbol, action, price, size, algo="", algo_param=None, userdata=""):
        if algo_param is None:
            algo_param = dict()
        
        # this order object is not for TradeApi, but for strategy itself to remember the order
        order = Order.new_order(symbol, action, price, size, self.ctx.trade_date, 0)
        order.entrust_no = self._get_next_num('entrust_no')
        
        task = Task(self._get_next_task_no(),
                    algo=algo, algo_param=algo_param,
                    data=order,
                    function_name="place_order")
        self.ctx.pm.add_task(task)
        # self.task_id_map[order.task_id].append(order.entrust_no)
        
        # self.pm.add_order(order)
        
        e = Event(EVENT_TYPE.PLACE_ORDER)
        e.dic['task'] = task
        self.put_to_gateway(e)
    
    def cancel_order(self, entrust_no):
        e = Event(EVENT_TYPE.CANCEL_ORDER)
        e.dic['entrust_no'] = entrust_no
        self.put_to_gateway(e)
    
    # ----------------------------------------------------------------------------------------
    # PMS
    
    def goal_portfolio(self, positions, algo="", algo_param=None, userdata=""):
        if algo_param is None:
            algo_param = dict()
        
        task = Task(self._get_next_task_no(), data=positions,
                    algo=algo, algo_param=algo_param,
                    function_name="goal_portfolio")
        self.ctx.pm.add_task(task)
        
        e = Event(EVENT_TYPE.GOAL_PORTFOLIO)
        e.dic['task'] = task
        self.put_to_gateway(e)
    
    # ----------------------------------------------------------------------------------------
    # query account, universe, position, portfolio
    
    def query_account(self, format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_ACCOUNT)
        e.dic['args'] = args
        self.put_to_gateway(e)
    
    def query_universe(self, format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_UNIVERSE)
        e.dic['args'] = args
        self.put_to_gateway(e)
    
    def query_position(self, mode="all", symbols="", format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_POSITION)
        e.dic['args'] = args
        self.put_to_gateway(e)
    
    def query_portfolio(self, format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_PORTFOLIO)
        e.dic['args'] = args
        self.put_to_gateway(e)
    
    # ----------------------------------------------------------------------------------------
    # query task, order, trade
    
    def query_task(self, task_id=-1, format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_TASK)
        e.dic['args'] = args
        self.put_to_gateway(e)
    
    def query_order(self, task_id=-1, format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_ORDER)
        e.dic['args'] = args
        self.put_to_gateway(e)
    
    def query_trade(self, task_id=-1, format=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_TRADE)
        e.dic['args'] = args
        self.put_to_gateway(e)


'''
'''
class BaseGateway(object):
    """
    Strategy communicates with Gateway using APIs defined by ourselves;
    Gateway communicates with brokers using brokers' APIs;
    Gateway can also communicate with simulator.
    
    Attributes
    ----------
    ctx : Context
        Trading context, including data_api, dataview, calendar, etc.

    Notes
    -----
    Gateway knows nothing about task_id but entrust_no, so does Simulator.

    """
    
    def __init__(self):
        super(BaseGateway, self).__init__()
        
        self.ctx = None
    
    @abstractmethod
    def init_from_config(self, props):
        pass
    
    def register_context(self, context=None):
        self.ctx = context
    
    def on_new_day(self, trade_date):
        pass
    
'''


class RealTimeTradeApi_async(BaseTradeApi, EventEngine):
    """
    Attributes
    ----------
    _trade_api : TradeApi
    
    """
    def __init__(self):
        super(RealTimeTradeApi_async, self).__init__()
        
        self.ctx = None
        
        self._trade_api = None

        self._task_no_id_map = dict()

        self.seq_gen = SequenceGenerator()
        
    def init_from_config(self, props):
        """
        Instantiate TradeAPI and login.
        
        Parameters
        ----------
        props : dict

        """
        if self._trade_api is not None:
            self._trade_api.close()
    
        def get_from_list_of_dict(l, key, default=None):
            res = None
            for dic in l:
                res = dic.get(key, None)
                if res is not None:
                    break
            if res is None:
                res = default
            return res
    
        props_default = dict()  # jutil.read_json(jutil.join_relative_path('etc/trade_config.json'))
        dic_list = [props, props_default]
    
        address = get_from_list_of_dict(dic_list, "remote.trade.address", "")
        username = get_from_list_of_dict(dic_list, "remote.trade.username", "")
        password = get_from_list_of_dict(dic_list, "remote.trade.password", "")
        if address is None or username is None or password is None:
            raise ValueError("no address, username or password available!")
    
        tapi = TradeApi(address)
        self.set_trade_api_callbacks(tapi)

        # 使用用户名、密码登陆， 如果成功，返回用户可用的策略帐号列表
        print("\n{}@{} login...".format(username, address))
        user_info, msg = tapi.login(username, password)
        print("    Login msg: {:s}".format(msg))
        print("    Login user info: {:s}\n".format(user_info))
        self._trade_api = tapi

        # event types and trade_api functions are one-to-one corresponded
        self._omni_api_map = {EVENT_TYPE.QUERY_ACCOUNT: self._trade_api.query_account,
                              EVENT_TYPE.QUERY_UNIVERSE: self._trade_api.query_universe,
                              EVENT_TYPE.QUERY_POSITION: self._trade_api.query_position,
                              EVENT_TYPE.QUERY_PORTFOLIO: self._trade_api.query_portfolio,
                              EVENT_TYPE.QUERY_TASK: self._trade_api.query_task,
                              EVENT_TYPE.QUERY_TRADE: self._trade_api.query_trade,
                              EVENT_TYPE.QUERY_ORDER: self._trade_api.query_order,
                              }
    
    # -------------------------------------------------------------------------------------------
    # On TradeAPI Callback: put a corresponding event to EventRealTimeInstance

    def set_trade_api_callbacks(self, trade_api):
        trade_api.set_task_status_callback(self.on_task_status)
        trade_api.set_order_status_callback(self.on_order_status)
        trade_api.set_trade_callback(self.on_trade)
        trade_api.set_connection_callback(self.on_connection_callback)

    def on_connection_callback(self, connected):
        """
        
        Parameters
        ----------
        connected : bool

        """
        if connected:
            print("TradeAPI connected.")
            event_type = EVENT_TYPE.TRADE_API_CONNECTED
        else:
            print("TradeAPI disconnected.")
            event_type = EVENT_TYPE.TRADE_API_DISCONNECTED
        e = Event(event_type)
        self.ctx.instance.put(e)

    def on_trade(self, ind_dic):
        """
        
        Parameters
        ----------
        ind_dic : dict

        """
        # print("\nGateway on trade: ")
        # print(ind_dic)
        if 'security' in ind_dic:
            ind_dic['symbol'] = ind_dic.pop('security')
        
        ind = Trade.create_from_dict(ind_dic)
        ind.task_no = self._task_no_id_map[ind.task_id]
        
        e = Event(EVENT_TYPE.TRADE_IND)
        e.dic['ind'] = ind
        self.ctx.instance.put(e)

    def on_order_status(self, ind_dic):
        """
        
        Parameters
        ----------
        ind_dic : dict

        """
        # print("\nGateway on order status: ")
        # print(ind_dic)
        if 'security' in ind_dic:
            ind_dic['symbol'] = ind_dic.pop('security')
        
        ind = OrderStatusInd.create_from_dict(ind_dic)
        ind.task_no = self._task_no_id_map[ind.task_id]
        
        e = Event(EVENT_TYPE.ORDER_STATUS_IND)
        e.dic['ind'] = ind
        self.ctx.instance.put(e)
        
    def on_task_status(self, ind_dic):
        # print("\nGateway on task ind: ")
        # print(ind_dic)
        ind = TaskInd.create_from_dict(ind_dic)
        ind.task_no = self._task_no_id_map[ind.task_id]

        e = Event(EVENT_TYPE.TASK_STATUS_IND)
        e.dic['ind'] = ind
        self.ctx.instance.put(e)

    def on_omni_call(self, event):
        func = self._omni_api_map.get(event.type_, None)
        if func is None:
            print("{} not recgonized. Ignore.".format(event))
            return
    
        args = event.dic.get('args', None)
        func(args)

    def on_goal_portfolio(self, event):
        task = event.dic['task']
        positions = task.data
    
        # TODO: compatibility
        for dic in positions:
            dic['symbol'] = dic.pop('security')
    
        task_id, msg = self._trade_api.goal_portfolio(positions, algo=task.algo, algo_param=task.algo_param)
    
        self._generate_on_task_rsp(task.task_no, task_id, msg)

    def _generate_on_task_rsp(self, task_no, task_id, msg):
        # this rsp is generated by gateway itself
        rsp = TaskRsp(task_no=task_no, task_id=task_id, msg=msg)
        if rsp.success:
            self._task_no_id_map[task_id] = task_no
    
        # DEBUG
        print("\nGateway generate task_rsp {}".format(rsp))
        e = Event(EVENT_TYPE.TASK_RSP)
        e.dic['rsp'] = rsp
        self.ctx.instance.put(e)

    def on_place_order(self, event):
        task = event.dic['task']
        order = task.data
        task_id, msg = self._trade_api.place_order(order.symbol, order.entrust_action,
                                                   order.entrust_price, order.entrust_size,
                                                   task.algo, task.algo_param)
    
        self._generate_on_task_rsp(task.task_no, task_id, msg)

    # -------------------------------------------------------------------------------------------
    # Run
    
    def run(self):
        """
        Listen to certain events and run the EventEngine.
        Events include:
            1. placement & cancellation of orders
            2. query of universe, account, position and portfolio
            3. etc.

        """
        for e_type in self._omni_api_map.keys():
            self.register(e_type, self.on_omni_call)
        
        self.register(EVENT_TYPE.PLACE_ORDER, self.on_place_order)
        self.register(EVENT_TYPE.CANCEL_ORDER, self.on_omni_call)
        self.register(EVENT_TYPE.GOAL_PORTFOLIO, self.on_goal_portfolio)
        
        self.start(timer=False)
    
    # -------------------------------------------------------------------------------------------
    # API
    
    def _get_next_num(self, key):
        """used to generate id for orders and trades."""
        return str(np.int64(self.ctx.trade_date) * 10000 + self.seq_gen.get_next(key))

    def _get_next_task_no(self):
        return self._get_next_num('task_no')

    def publish_event(self, event):
        self.put(event)

    # ----------------------------------------------------------------------------------------
    # place & cancel

    def place_order(self, symbol, action, price, size, algo="", algo_param=None, userdata=""):
        if algo_param is None:
            algo_param = dict()
    
        # this order object is not for TradeApi, but for strategy itself to remember the order
        order = Order.new_order(symbol, action, price, size, self.ctx.trade_date, 0)
        order.entrust_no = self._get_next_num('entrust_no')
    
        task = Task(self._get_next_task_no(),
                    algo=algo, algo_param=algo_param,
                    data=order,
                    function_name="place_order")
        self.ctx.pm.add_task(task)
        # self.task_id_map[order.task_id].append(order.entrust_no)
    
        # self.pm.add_order(order)
    
        e = Event(EVENT_TYPE.PLACE_ORDER)
        e.dic['task'] = task
        self.publish_event(e)

    def cancel_order(self, entrust_no):
        e = Event(EVENT_TYPE.CANCEL_ORDER)
        e.dic['entrust_no'] = entrust_no
        self.publish_event(e)

    # ----------------------------------------------------------------------------------------
    # PMS

    def goal_portfolio(self, positions, algo="", algo_param=None, userdata=""):
        if algo_param is None:
            algo_param = dict()
    
        task = Task(self._get_next_task_no(), data=positions,
                    algo=algo, algo_param=algo_param,
                    function_name="goal_portfolio")
        self.ctx.pm.add_task(task)
    
        e = Event(EVENT_TYPE.GOAL_PORTFOLIO)
        e.dic['task'] = task
        self.publish_event(e)

    # ----------------------------------------------------------------------------------------
    # query account, universe, position, portfolio

    def query_account_async(self, userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_ACCOUNT)
        e.dic['args'] = args
        self.publish_event(e)

    def query_universe_async(self, userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_UNIVERSE)
        e.dic['args'] = args
        self.publish_event(e)

    def query_position_async(self, mode="all", symbols="", userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_POSITION)
        e.dic['args'] = args
        e.dic['securities'] = e.dic.pop['symbols']
        self.publish_event(e)

    def query_portfolio_async(self, userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_PORTFOLIO)
        e.dic['args'] = args
        self.publish_event(e)
    
    def query_account(self, format=""):
        return self._trade_api.query_account(format=format)

    def query_universe(self, format=""):
        return self._trade_api.query_universe(format=format)

    def query_position(self, mode = "all", symbols = "", format=""):
        return self._trade_api.query_position(mode=mode, securities=symbols, format=format)

    def query_portfolio(self, format=""):
        return self._trade_api.query_portfolio(format=format)

    # ----------------------------------------------------------------------------------------
    # Use Strategy

    def use_strategy(self, strategy_id) :
        return self._trade_api.use_strategy(strategy_id)
    
    # ----------------------------------------------------------------------------------------
    # query task, order, trade

    def query_task(self, task_id=-1, userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_TASK)
        e.dic['args'] = args
        self.publish_event(e)

    def query_order(self, task_id=-1, userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_ORDER)
        e.dic['args'] = args
        self.publish_event(e)

    def query_trade(self, task_id=-1, userdata=""):
        args = locals()
        e = Event(EVENT_TYPE.QUERY_TRADE)
        e.dic['args'] = args
        self.publish_event(e)


class RealTimeTradeApi(TradeApi):
    def __init__(self, props):
        address = props['remote.trade.address']
        
        super(RealTimeTradeApi, self).__init__(address)
        
        self.ctx = None
        self.user_info = dict()
        
    def init_from_config(self, props):
        self.set_trade_api_callbacks()

        def get_from_list_of_dict(l, key, default=None):
            res = None
            for dic in l:
                res = dic.get(key, None)
                if res is not None:
                    break
            if res is None:
                res = default
            return res

        props_default = dict()  # jutil.read_json(jutil.join_relative_path('etc/trade_config.json'))
        dic_list = [props, props_default]

        address = get_from_list_of_dict(dic_list, "remote.trade.address", "")
        username = get_from_list_of_dict(dic_list, "remote.trade.username", "")
        password = get_from_list_of_dict(dic_list, "remote.trade.password", "")
        if address is None or username is None or password is None:
            raise ValueError("no address, username or password available!")
        
        # 使用用户名、密码登陆， 如果成功，返回用户可用的策略帐号列表
        print("\nTradeApi login {}@{}".format(username, address))
        user_info, msg = self.login(username, password)
        if not (msg == '0,'):
            print("    login failed: msg = '{}'\n".format(msg))
        else:
            print("    login success. user info: \n"
                  "    {:s}\n".format(user_info))
            
        self.user_info = user_info
        
        strategy_no = get_from_list_of_dict(dic_list, "strategy.no", 0)
        self.use_strategy(strategy_no)
    
    def set_trade_api_callbacks(self):
        self.set_task_callback(self.on_task_status)
        self.set_ordstatus_callback(self.on_order_status)
        self.set_trade_callback(self.on_trade)
        self.set_connection_callback(self.on_connection_callback)

    def on_connection_callback(self, connected):
        """
        
        Parameters
        ----------
        connected : bool

        """
        if connected:
            print("TradeAPI connected.")
        else:
            print("TradeAPI disconnected.")

    def on_trade(self, ind_dic):
        """
        
        Parameters
        ----------
        ind_dic : dict

        """
        # print("\nGateway on trade: ")
        # print(ind_dic)
        if 'security' in ind_dic:
            ind_dic['symbol'] = ind_dic.pop('security')
        
        ind = Trade.create_from_dict(ind_dic)
        
        self.ctx.strategy.on_trade(ind)
    
    def on_order_status(self, ind_dic):
        """
        
        Parameters
        ----------
        ind_dic : dict

        """
        # print("\nGateway on order status: ")
        # print(ind_dic)
        if 'security' in ind_dic:
            ind_dic['symbol'] = ind_dic.pop('security')
        
        ind = OrderStatusInd.create_from_dict(ind_dic)
        
        self.ctx.strategy.on_order_status(ind)
    
    def on_task_status(self, ind_dic):
        # print("\nGateway on task ind: ")
        # print(ind_dic)
        ind = TaskInd.create_from_dict(ind_dic)
        
        self.ctx.strategy.on_task_status(ind)

    @staticmethod
    def _is_failed_task(task_id):
        return (task_id is None) or (task_id == 0)
    
    def place_order(self, symbol, action, price, size, algo="", algo_param={}, userdata=""):
        # Generate Task
        order = Order.new_order(symbol, action, price, size, self.ctx.trade_date, self.ctx.time,
                                order_type=common.ORDER_TYPE.LIMIT)
        
        task_id, msg = super(RealTimeTradeApi, self).place_order(symbol, action, price, size, algo, algo_param, userdata)
        if self._is_failed_task(task_id):
            return task_id, msg
    
        task = Task(task_id,
                    algo=algo, algo_param=algo_param, data=order,
                    function_name='place_order')
        
        self.ctx.pm.add_task(task)
    
        return task_id, msg
    

# ---------------------------------------------
# For Alpha Strategy

class AlphaTradeApi(BaseTradeApi):
    def __init__(self):
        super(AlphaTradeApi, self).__init__()
        self.ctx = None
        
        self._simulator = DailyStockSimulator()
        self.entrust_no_task_id_map = dict()
        self.seq_gen = SequenceGenerator()

        self.commission_rate = 0.0
        
        self.MATCH_TIME = 143000

    def _get_next_task_id(self):
        return np.int64(self.ctx.trade_date) * 10000 + self.seq_gen.get_next('task_id')

    def _add_task_id(self, ind):
        no = ind.entrust_no
        task_id = self.entrust_no_task_id_map[no]
        ind.task_id = task_id
        # ind.task_no = task_id

    def init_from_config(self, props):
        self.commission_rate = props.get('commission_rate', 0.0)
        
        self.set_order_status_callback(lambda ind: self.ctx.strategy.on_order_status(ind))
        self.set_trade_callback(lambda ind: self.ctx.strategy.on_trade(ind))
        self.set_task_status_callback(lambda ind: self.ctx.strategy.on_task_status(ind))
        
    def query_account(self, format=""):
        pass
    
    def on_new_day(self, trade_date):
        self._simulator.on_new_day(trade_date)

    def on_after_market_close(self):
        self._simulator.on_after_market_close()

    def place_order(self, security, action, price, size, algo="", algo_param={}, userdata=""):
        if size <= 0:
            print("Invalid size {}".format(size))
            return
    
        # Generate Task
        order = Order.new_order(security, action, price, size, self.ctx.trade_date, self.ctx.time,
                                order_type=common.ORDER_TYPE.LIMIT)
    
        task_id = self._get_next_task_id()
        order.task_id = task_id
    
        task = Task(task_id,
                    algo=algo, algo_param=algo_param, data=order,
                    function_name='place_order', trade_date=self.ctx.trade_date)
        # task.task_no = task_id
    
        # Send Order to Exchange
        entrust_no = self._simulator.add_order(order)
        task.data.entrust_no = entrust_no
    
        self.ctx.pm.add_task(task)
        self.entrust_no_task_id_map[entrust_no] = task.task_id
    
        order_status_ind = OrderStatusInd(order)
        order_status_ind.order_status = common.ORDER_STATUS.ACCEPTED
        self._order_status_callback(order_status_ind)
    
        return task_id, ""

    def cancel_order(self, task_id):
        task = self.ctx.pm.get_task(task_id)
        if task.function_name == 'place_order':
            order = task.data
            entrust_no = order.entrust_no
            order_status_ind, err_msg = self._simulator.cancel_order(entrust_no)
            task_id = self.entrust_no_task_id_map[entrust_no]
            order_status_ind.task_id = task_id
            # order_status_ind.task_no = task_id
            self._order_status_callback(order_status_ind)
        else:
            raise NotImplementedError("cancel task with function_name = {}".format(task.function_name))

    def goal_portfolio(self, positions, algo="", algo_param={}, userdata=""):
        # Generate Orders
        task_id = self._get_next_task_id()
        
        orders = []
        for goal in positions:
            sec, goal_size = goal['symbol'], goal['size']
            if sec in self.ctx.pm.holding_securities:
                current_size = self.ctx.pm.get_position(sec).current_size
            else:
                current_size = 0
            diff_size = goal_size - current_size
            if diff_size != 0:
                action = common.ORDER_ACTION.BUY if diff_size > 0 else common.ORDER_ACTION.SELL
        
                order = FixedPriceTypeOrder.new_order(sec, action, 0.0, abs(diff_size), self.ctx.trade_date, 0)
                if algo == 'vwap':
                    order.price_target = 'vwap'  # TODO
                elif algo == '':
                    order.price_target = 'vwap'
                else:
                    raise NotImplementedError("goal_portfolio algo = {}".format(algo))

                order.task_id = task_id
                orders.append(order)

        # Generate Task
        task = Task(task_id,
                    algo=algo, algo_param=algo_param, data=orders,
                    function_name='goal_portfolio', trade_date=self.ctx.trade_date)

        self.ctx.pm.add_task(task)

        # Send Orders to Exchange
        for order in orders:
            entrust_no = self._simulator.add_order(order)
            # task.data.entrust_no = entrust_no
            self.entrust_no_task_id_map[entrust_no] = task.task_id
            
            order_status_ind = OrderStatusInd(order)
            order_status_ind.order_status = common.ORDER_STATUS.ACCEPTED
            self._order_status_callback(order_status_ind)
    
    def goal_portfolio_by_batch_order(self, goals):
        assert len(goals) == len(self.ctx.universe)
    
        orders = []
        for goal in goals:
            sec, goal_size = goal.symbol, goal.size
            if sec in self.ctx.pm.holding_securities:
                current_size = self.ctx.pm.get_position(sec).current_size
            else:
                current_size = 0
            diff_size = goal_size - current_size
            if diff_size != 0:
                action = common.ORDER_ACTION.BUY if diff_size > 0 else common.ORDER_ACTION.SELL
            
                order = FixedPriceTypeOrder.new_order(sec, action, 0.0, abs(diff_size), self.ctx.trade_date, 0)
                order.price_target = 'vwap'  # TODO
            
                orders.append(order)
        
        for order in orders:
            self._simulator.add_order(order)

    @property
    def match_finished(self):
        return self._simulator.match_finished
    
    '''
    @abstractmethod
    def match(self, price_dict, time=0):
        """
        Match un-fill orders in simulator. Return trade indications.

        Parameters
        ----------
        price_dict : dict
        time : int
        # TODO: do we need time parameter?

        Returns
        -------
        list

        """
        return self._simulator.match(price_dict, date=self.ctx.trade_date, time=time)

    '''
    def calc_commission(self, trade_ind):
        to = abs(trade_ind.fill_price * trade_ind.fill_size)
        res = to * self.commission_rate
        return res

    def _add_commission(self, ind):
        comm = self.calc_commission(ind)
        ind.commission = comm
        
    def match_and_callback(self, price_dict):
        results = self._simulator.match(price_dict, date=self.ctx.trade_date, time=self.MATCH_TIME)
    
        for trade_ind, order_status_ind in results:
            self._add_commission(trade_ind)
        
            task_id = self.entrust_no_task_id_map[trade_ind.entrust_no]
            self._add_task_id(trade_ind)
            self._add_task_id(order_status_ind)
        
            self._order_status_callback(order_status_ind)
            self._trade_callback(trade_ind)
        
            task = self.ctx.pm.get_task(task_id)
            if task.is_finished:
                task_ind = TaskInd(task_id, task_status=task.task_status,
                                   task_algo='', task_msg="")
                self._task_status_callback(task_ind)
        return results


class DailyStockSimulator(object):
    """This is not event driven!

    Attributes
    ----------
    __orders : list of Order
        Store orders that have not been filled.

    """
    
    def __init__(self):
        # TODO heap is better for insertion and deletion. We only need implement search of heapq module.
        self.__orders = dict()
        self.seq_gen = SequenceGenerator()
        
        self.date = 0
        self.time = 0
    
    def on_new_day(self, trade_date):
        self.date = trade_date
    
    def on_after_market_close(self):
        # self._refresh_orders() #TODO sometimes we do not want to refresh (multi-days match)
        pass
    
    def _refresh_orders(self):
        self.__orders.clear()
    
    def _next_fill_no(self):
        return str(np.int64(self.date) * 10000 + self.seq_gen.get_next('fill_no'))
    
    @property
    def match_finished(self):
        return len(self.__orders) == 0
    
    @staticmethod
    def _validate_order(order):
        # TODO to be enhanced
        assert order is not None
    
    @staticmethod
    def _validate_price(price_dic):
        # TODO to be enhanced
        assert price_dic is not None

    def _get_next_entrust_no(self):
        """used to generate id for orders and trades."""
        return str(self.seq_gen.get_next('entrust_no'))
    
    def add_order(self, order):
        """
        Add one order to the simulator.

        Parameters
        ----------
        order : Order

        Returns
        -------
        err_msg : str
            default ""

        """
        neworder = copy.copy(order)
        self._validate_order(order)

        entrust_no = self._get_next_entrust_no()
        neworder.entrust_no = entrust_no

        self.__orders[entrust_no] = neworder
        return entrust_no
    
    def cancel_order(self, entrust_no):
        """
        Cancel an order.

        Parameters
        ----------
        entrust_no : str

        Returns
        -------
        err_msg : str
            default ""

        """
        order = self.__orders.pop(entrust_no, None)
        if order is None:
            err_msg = "No order with entrust_no {} in simulator.".format(entrust_no)
            order_status_ind = None
        else:
            order.cancel_size = order.entrust_size - order.fill_size
            order.order_status = common.ORDER_STATUS.CANCELLED
            
            err_msg = ""
            order_status_ind = OrderStatusInd(order)
        return order_status_ind, err_msg
    
    def match(self, price_dic, date=19700101, time=150000):
        self._validate_price(price_dic)
        
        results = []
        for order in self.__orders.values():
            symbol = order.symbol
            symbol_dic = price_dic[symbol]
            
            # get fill price
            if isinstance(order, FixedPriceTypeOrder):
                price_target = order.price_target
                fill_price = symbol_dic[price_target]
            elif isinstance(order, VwapOrder):
                if order.start != -1:
                    raise NotImplementedError("Vwap of a certain time range")
                fill_price = symbol_dic['vwap']
            elif isinstance(order, Order):
                # TODO
                fill_price = symbol_dic['close']
            else:
                raise NotImplementedError("order class {} not support!".format(order.__class__))
            
            # get fill size
            fill_size = order.entrust_size - order.fill_size
            
            # create trade indication
            trade_ind = Trade(order)
            trade_ind.set_fill_info(fill_price, fill_size,
                                    date, time,
                                    self._next_fill_no())
            
            # update order status
            order.fill_price = (order.fill_price * order.fill_size
                                + fill_price * fill_size) / (order.fill_size + fill_size)
            order.fill_size += fill_size
            if order.fill_size == order.entrust_size:
                order.order_status = common.ORDER_STATUS.FILLED
                
            order_status_ind = OrderStatusInd(order)
            
            results.append((trade_ind, order_status_ind))
        
        self.__orders = {k: v for k, v in self.__orders.items() if not v.is_finished}
        # self.cancel_order(order.entrust_no)  # TODO DEBUG
        
        return results


# ---------------------------------------------
# For Event-driven Strategy

class OrderBook(object):
    def __init__(self):
        self.orders = dict()
        
        self.seq_gen = SequenceGenerator()
        self.participation_rate = 1.0
    
    def _next_fill_no(self):
        return str(self.seq_gen.get_next('trade_id'))
    
    def _next_order_entrust_no(self):
        return str(self.seq_gen.get_next('order_id'))
    
    def add_order(self, order):
        neworder = copy.copy(order)
        
        entrust_no = self._next_order_entrust_no()
        neworder.entrust_no = entrust_no
        
        self.orders[entrust_no] = neworder
        
        return entrust_no
    
    def _make_tick_trade(self, quote):
        raise NotImplementedError()
    
    def make_trade(self, quote, freq):
        
        if freq == common.QUOTE_TYPE.TICK:
            # TODO
            return self._make_tick_trade(quote)
        
        elif (freq == common.QUOTE_TYPE.MIN
              or freq == common.QUOTE_TYPE.FIVEMIN
              or freq == common.QUOTE_TYPE.QUARTERMIN
              or freq == common.QUOTE_TYPE.SPECIALBAR):
            return self._make_trade_bar(quote)
        
        elif freq == common.QUOTE_TYPE.DAILY:
            return self._make_trade_bar(quote)
    
    def _make_trade_bar(self, quote_dic):
        
        result = []
        
        for entrust_no, order in self.orders.items():
            quote = quote_dic[order.symbol]
            low = quote.low
            high = quote.high
            quote_date = quote.trade_date
            quote_time = quote.time
            volume = quote.volume
            
            '''
            if order.order_type == common.ORDER_TYPE.LIMIT:
                if order.entrust_action == common.ORDER_ACTION.BUY and order.entrust_price >= low:
                    trade = Trade()
                    trade.init_from_order(order)
                    trade.send_fill_info(order.entrust_price, order.entrust_size,
                                         quote_date, quote_time,
                                         self._next_fill_no())
                    
                    order.order_status = common.ORDER_STATUS.FILLED
                    order.fill_size = trade.fill_size
                    order.fill_price = trade.fill_price
                    
                    orderstatus_ind = OrderStatusInd()
                    orderstatus_ind.init_from_order(order)
                    
                    result.append((trade, orderstatus_ind))
                    
                elif order.entrust_action == common.ORDER_ACTION.SELL and order.entrust_price <= high:
                    trade = Trade()
                    trade.init_from_order(order)
                    trade.send_fill_info(order.entrust_price, order.entrust_size,
                                         quote_date, quote_time,
                                         self._next_fill_no())
                    
                    order.order_status = common.ORDER_STATUS.FILLED
                    order.fill_size = trade.fill_size
                    order.fill_price = trade.fill_price
                    
                    orderstatus_ind = OrderStatusInd()
                    orderstatus_ind.init_from_order(order)

                    result.append((trade, orderstatus_ind))
            
            elif order.order_type == common.ORDER_TYPE.STOP:
                if order.entrust_action == common.ORDER_ACTION.BUY and order.entrust_price <= high:
                    trade = Trade()
                    trade.init_from_order(order)
                    trade.send_fill_info(order.entrust_price, order.entrust_size,
                                         quote_date, quote_time,
                                         self._next_fill_no())
                    
                    order.order_status = common.ORDER_STATUS.FILLED
                    order.fill_size = trade.fill_size
                    order.fill_price = trade.fill_price
                    orderstatus_ind = OrderStatusInd()
                    orderstatus_ind.init_from_order(order)
                    result.append((trade, orderstatus_ind))
                
                if order.entrust_action == common.ORDER_ACTION.SELL and order.entrust_price >= low:
                    trade = Trade()
                    trade.init_from_order(order)
                    trade.send_fill_info(order.entrust_price, order.entrust_size,
                                         quote_date, quote_time,
                                         self._next_fill_no())
                    
                    order.order_status = common.ORDER_STATUS.FILLED
                    order.fill_size = trade.fill_size
                    order.fill_price = trade.fill_price
                    orderstatus_ind = OrderStatusInd()
                    orderstatus_ind.init_from_order(order)
                    result.append((trade, orderstatus_ind))
            '''
            
            entrust_price = order.entrust_price
            entrust_size = order.entrust_size
            
            fill_size = 0
            if order.order_type == common.ORDER_TYPE.LIMIT:
                if common.ORDER_ACTION.is_positive(order.entrust_action) and entrust_price >= low:
                    fill_price = min(entrust_price, high)
                    # fill_size = min(entrust_size, self.participation_rate * volume)
                    fill_size = entrust_size
                    
                elif common.ORDER_ACTION.is_negative(order.entrust_action) and order.entrust_price <= high:
                    fill_price = max(entrust_price, low)
                    # fill_size = min(entrust_size, self.participation_rate * volume)
                    fill_size = entrust_size

            elif order.order_type == common.ORDER_TYPE.STOP:
                if common.ORDER_ACTION.is_positive(order.entrust_action) and order.entrust_price <= high:
                    fill_price = max(entrust_price, low)
                    # fill_size = min(entrust_size, self.participation_rate * volume)
                    fill_size = entrust_size

                if common.ORDER_ACTION.is_negative(order.entrust_action) and order.entrust_price >= low:
                    fill_price = min(entrust_price, high)
                    # fill_size = min(entrust_size, self.participation_rate * volume)
                    fill_size = entrust_size
            
            elif order.order_type == common.ORDER_TYPE.VWAP:
                fill_price = quote.vwap
                fill_size = entrust_size

            if not fill_size:
                continue
                
            trade_ind = Trade(order)
            trade_ind.set_fill_info(order.entrust_price, order.entrust_size,
                                    quote_date, quote_time,
                                    self._next_fill_no())
            
            order.fill_price = ((order.fill_price * order.fill_size + fill_size * fill_price)
                                / (order.fill_size + fill_size))
            order.fill_size += fill_size
            if order.fill_size == order.entrust_size:
                order.order_status = common.ORDER_STATUS.FILLED
            
            order_status_ind = OrderStatusInd(order)
            
            result.append((trade_ind, order_status_ind))
            
        self.orders = {k: v for k, v in self.orders.items() if not v.is_finished}

        return result
    
    def cancel_order(self, entrust_no):
        order = self.orders.pop(entrust_no)
        order.cancel_size = order.entrust_size - order.fill_size
        order.order_status = common.ORDER_STATUS.CANCELLED
        
        order_status_ind = OrderStatusInd(order)
        
        '''
        for i in xrange(len(self.orders)):
            order = self.orders[i]
            
            if (order.is_finished):
                continue
            
            if (order.entrust_no == entrust_no):
                order.cancel_size = order.entrust_size - order.fill_size
                order.order_status = common.ORDER_STATUS.CANCELLED
            
            # todo
            orderstatus = OrderStatusInd()
            orderstatus.init_from_order(order)
            
            return orderstatus
    
        '''
        return order_status_ind
        
    '''
    def cancel_all(self):
        result = []
        for order in self.orders:
            if order.is_finished:
                continue
            order.cancel_size = order.entrust_size - order.fill_size
            order.order_status = common.ORDER_STATUS.CANCELLED
            
            # todo
            orderstatus = OrderStatusInd()
            orderstatus.init_from_order(order)
            result.append(orderstatus)
        
        return result

    '''


class BacktestTradeApi(BaseTradeApi):
    def __init__(self):
        super(BacktestTradeApi, self).__init__()
        
        self.ctx = None
        
        self._orderbook = OrderBook()
        self.seq_gen = SequenceGenerator()
        self.entrust_no_task_id_map = dict()
        
        self.commission_rate = 0.0
        
    def _get_next_num(self, key):
        """used to generate id for orders and trades."""
        return str(np.int64(self.ctx.trade_date) * 10000 + self.seq_gen.get_next(key))

    def _get_next_task_id(self):
        return np.int64(self.ctx.trade_date) * 10000 + self.seq_gen.get_next('task_id')
    
    def init_from_config(self, props):
        self.commission_rate = props.get('commission_rate', 0.0)
        
        self.set_order_status_callback(lambda ind: self.ctx.strategy.on_order_status(ind))
        self.set_trade_callback(lambda ind: self.ctx.strategy.on_trade(ind))
        self.set_task_status_callback(lambda ind: self.ctx.strategy.on_task_status(ind))

    def on_new_day(self, trade_date):
        self._orderbook = OrderBook()
    
    def use_strategy(self, strategy_id):
        pass
    
    def place_order(self, security, action, price, size, algo="", algo_param={}, userdata=""):
        if size <= 0:
            print("Invalid size {}".format(size))
            return
        
        # Generate Order
        if algo == 'vwap':
            order_type = common.ORDER_TYPE.VWAP
        else:
            order_type = common.ORDER_TYPE.LIMIT
        order = Order.new_order(security, action, price, size, self.ctx.trade_date, self.ctx.time,
                                order_type=order_type)

        # Generate Task
        task_id = self._get_next_task_id()
        order.task_id = task_id
        
        task = Task(task_id,
                    algo=algo, algo_param=algo_param, data=order,
                    function_name='place_order', trade_date=self.ctx.trade_date)
        # task.task_no = task_id
        
        # Send Order to Exchange
        entrust_no = self._orderbook.add_order(order)
        task.data.entrust_no = entrust_no
        
        self.ctx.pm.add_task(task)
        self.entrust_no_task_id_map[entrust_no] = task.task_id

        order_status_ind = OrderStatusInd(order)
        order_status_ind.order_status = common.ORDER_STATUS.ACCEPTED
        # order_status_ind.task_no = task_id
        self._order_status_callback(order_status_ind)


        '''
        # TODO: not necessary
        rsp = OrderRsp(entrust_no=entrust_no, task_id=task_id, msg="")
        self.ctx.instance.strategy.on_order_rsp(rsp)
        '''
        
        return task_id, ""

    def cancel_order(self, task_id):
        task = self.ctx.pm.get_task(task_id)
        if task.function_name == 'place_order':
            order = task.data
            entrust_no = order.entrust_no
            order_status_ind = self._orderbook.cancel_order(entrust_no)
            task_id = self.entrust_no_task_id_map[entrust_no]
            order_status_ind.task_id = task_id
            # order_status_ind.task_no = task_id
            self._order_status_callback(order_status_ind)
        else:
            raise NotImplementedError("cancel task with function_name = {}".format(task.function_name))
    
    def _process_quote(self, df_quote, freq):
        return self._orderbook.make_trade(df_quote, freq)

    def calc_commission(self, trade_ind):
        to = abs(trade_ind.fill_price * trade_ind.fill_size)
        res = to * self.commission_rate
        return res
    
    def _add_task_id(self, ind):
        no = ind.entrust_no
        task_id = self.entrust_no_task_id_map[no]
        ind.task_id = task_id
        # ind.task_no = task_id
    
    def _add_commission(self, ind):
        comm = self.calc_commission(ind)
        ind.commission = comm
        
    def match_and_callback(self, quote, freq):
        results = self._process_quote(quote, freq)
        
        for trade_ind, order_status_ind in results:
            self._add_commission(trade_ind)
            
            # self._add_task_id(trade_ind)
            # self._add_task_id(order_status_ind)
            task_id = self.entrust_no_task_id_map[trade_ind.entrust_no]

            self._order_status_callback(order_status_ind)
            self._trade_callback(trade_ind)

            task = self.ctx.pm.get_task(task_id)
            if task.is_finished:
                task_ind = TaskInd(task_id, task_status=task.task_status,
                                   task_algo='', task_msg="")
                self._task_status_callback(task_ind)




import zmq
import time
import random
import Queue
import threading
import msgpack
import snappy
import copy

qEmpty = copy.copy(Queue.Empty)


def _unpack(str) :

    if str[0] == 'S':
        tmp = snappy.uncompress(str[1:])
        #print "SNAPPY: ", len(str), len(tmp)
        obj = msgpack.loads(tmp)
    elif str[0] == '\0':
        obj = msgpack.loads(str[1:])
    else:
        return None

    return obj

def _pack(obj) :
    #print "pack", obj
    tmp = msgpack.dumps(obj)    
    if len(tmp) > 1000:
        return 'S'  + snappy.compress(tmp)
    else:
        return '\0' + tmp

class JRpcClient :
    
    def __init__(self) :
        self._waiter_lock = threading.Lock()        
        self._waiter_map = {}

        self._should_close = False
        self._next_callid = 0
        self._send_lock = threading.Lock()
        self._callid_lock = threading.Lock()

        self._last_heartbeat_rsp_time = 0
        self._connected = False

        self.on_disconnected = None
        self.on_rpc_callback = None
        self._callback_queue = Queue.Queue()
        self._call_wait_queue = Queue.Queue()

        self._ctx = zmq.Context()
        self._pull_sock = self._ctx.socket(zmq.PULL)
        self._pull_sock.bind("inproc://pull_sock")
        self._push_sock = self._ctx.socket(zmq.PUSH)
        self._push_sock.connect("inproc://pull_sock")

        self._heartbeat_interval = 1
        self._heartbeat_timeout = 3

        self._addr = None

        t = threading.Thread(target=self._recv_run)
        t.setDaemon(True)
        t.start()
        self._recv_thread = t

        t = threading.Thread(target=self._callback_run)
        t.setDaemon(True)
        t.start()
        self._callback_thread = t
        
    def __del__(self):
        self.close()


    def next_callid(self):
        self._callid_lock.acquire()
        self._next_callid += 1
        callid = self._next_callid
        self._callid_lock.release()
        return callid

    def set_heartbeat_options(self, interval, timeout):
        self._heartbeat_interval = interval
        self._heartbeat_timeout = timeout

    def _recv_run(self):

        heartbeat_time = 0

        poller = zmq.Poller()
        poller.register(self._pull_sock, zmq.POLLIN)

        remote_sock = None

        while not self._should_close:

            try:
                if self._connected and time.time() - self._last_heartbeat_rsp_time > self._heartbeat_timeout:
                    self._connected = False
                    if self.on_disconnected: self._async_call(self.on_disconnected)

                if remote_sock and time.time() - heartbeat_time > self._heartbeat_interval :
                    self._send_hearbeat()
                    heartbeat_time = time.time()

                socks = dict(poller.poll(500))
                if self._pull_sock in socks and socks[self._pull_sock] == zmq.POLLIN:
                    cmd = self._pull_sock.recv()
                    if cmd == "CONNECT":
                        # print time.ctime(), "CONNECT " + self._addr
                        if remote_sock:
                            poller.unregister(remote_sock)
                            remote_sock.close()
                            remote_sock = None

                        remote_sock = self._do_connect()

                        if remote_sock :
                            poller.register(remote_sock, zmq.POLLIN)

                    elif cmd.startswith("SEND:") and remote_sock :
                        #print time.ctime(), "SEND " + cmd[5:]
                        remote_sock.send(cmd[5:])

                if remote_sock and remote_sock in socks and socks[remote_sock] == zmq.POLLIN:
                    data = remote_sock.recv()
                    if data:
                        #if not data.find("heartbeat"):
                        #    print time.ctime(), "RECV", data
                        self._on_data_arrived(str(data))

            except zmq.error.Again, e:
                #print "RECV timeout: ", e
                pass
            except Exception, e:
                print("_recv_run:", e)

    def _callback_run(self):
        while not self._should_close:
            try:
                r = self._callback_queue.get(timeout = 1)
                if r :
                    r()
            except qEmpty as e:
                pass
            except TypeError as e:
                if str(e) == "'NoneType' object is not callable":
                    pass
                else:
                    print "_callback_run {}".format(r), type(e), e
            except Exception as e:
                print "_callback_run {}".format(r), type(e), e

    def _async_call(self, func):
        self._callback_queue.put( func )

    def _send_request(self, json) :

        try:
            self._send_lock.acquire()
            self._push_sock.send("SEND:" + json)

        finally:
            self._send_lock.release()
            
    def connect(self, addr) :
        self._addr = addr
        self._push_sock.send("CONNECT")


    def _do_connect(self):

        client_id = str(random.randint(1000000, 100000000))

        socket = self._ctx.socket(zmq.DEALER)
        socket.identity = str(client_id) + '$' + str(random.randint(1000000, 1000000000))
        socket.setsockopt(zmq.RCVTIMEO, 500)
        socket.setsockopt(zmq.SNDTIMEO, 500)
        socket.setsockopt(zmq.LINGER, 0)
        socket.connect(self._addr)

        return socket

    def close(self):
        self._should_close = True
        self._callback_thread.join()
        self._recv_thread.join()
        
    def _on_data_arrived(self, str):
        try:
            msg = _unpack(str)
            #print "RECV", msg

            if not msg:
                print "wrong message format"
                return
                
            if msg.has_key('method') and msg['method'] == '.sys.heartbeat':
                self._last_heartbeat_rsp_time = time.time()
                if not self._connected:
                    self._connected = True
                    if self.on_connected :
                        self._async_call(self.on_connected)
                
                # Let user has a chance to check message in .sys.heartbeat
                if msg.has_key('result') and self.on_rpc_callback :
                    self._async_call( lambda: self.on_rpc_callback(msg['method'], msg['result']) )
                
            elif msg.has_key('id') and msg['id'] :

                # Call result
                id = int(msg['id'])
                
                if self._waiter_lock.acquire():
                    if self._waiter_map.has_key(id):
                        q = self._waiter_map[id]
                        if q: q.put(msg)
                    self._waiter_lock.release()
            else:
                # Notification message
                if msg.has_key('method') and msg.has_key('result') and self.on_rpc_callback :
                    self._async_call( lambda: self.on_rpc_callback(msg['method'], msg['result']) )
                
        except Exception, e:
            print( "_on_data_arrived:", e)
            pass
    

    def _send_hearbeat(self):
        msg = { 'jsonrpc' : '2.0',
                'method'  : '.sys.heartbeat',
                'params'  : { 'time': time.time() },
                'id'      : str(self.next_callid()) }
        json_str = _pack(msg)
        self._send_request(json_str)

    def _alloc_wait_queue(self):
        self._waiter_lock.acquire()
        if self._call_wait_queue:
            q = self._call_wait_queue
            self._call_wait_queue = None
        else:
            q = Queue.Queue()
        self._waiter_lock.release()
        return q

    def _free_wait_queue(self, q):
        self._waiter_lock.acquire()
        if not self._call_wait_queue :
            self._call_wait_queue  = q
        else:
            del q
        self._waiter_lock.release()

    def call(self, method, params, timeout = 6) :
        #print "call", method, params, timeout
        callid = self.next_callid()
        if timeout:

            q = self._alloc_wait_queue()

            self._waiter_lock.acquire()
            self._waiter_map[callid] = q
            self._waiter_lock.release()
        
        msg = { 'jsonrpc' : '2.0',
                'method'  : method,
                'params'  : params,
                'id'      : str(callid) }

        #print "SEND", msg
        json_str = _pack(msg)
        self._send_request(json_str)
        
        if timeout:
            ret = {}
            try:
                r = q.get(timeout = timeout)
                q.task_done()
            except Queue.Empty :
                r = None

            self._waiter_lock.acquire()
            self._waiter_map[callid] = None
            self._waiter_lock.release()
            self._free_wait_queue(q)

            if r:
                if r.has_key('result'):
                    ret['result'] = r['result']

                if r.has_key('error'):
                    ret['error'] = r['error']

            return ret if ret else { 'error': {'error': -1, 'message': "timeout"}}
        else:
            return { 'result': True }



# encoding: utf-8

import numpy as np
import pandas as pd
import jaqs.util as jutil


class RegisteredFunction(object):
    def __init__(self, func, name="", options=None):
        self.func = func
        self.name = name
        if not options:
            options = dict()
        self.options = options


class Context(object):
    """
    Used to store relevant context of the strategy.

    Attributes
    ----------
    data_api : DataService
        Data provider for the strategy.
    dataview : DataView
    gateway : gateway.Gateway object
        Broker of the strategy.
    universe : list of str
        Securities that the strategy cares about.
    _calendar : Calendar
        A certain calendar that the strategy refers to.
    snapshot : pd.DataFrame
        Current snapshot of data.

    Methods
    -------
    init_universe(univ)
        Add new securities.

    """
    def __init__(self, data_api=None, trade_api=None, gateway=None,
                 dataview=None,
                 strategy=None, pm=None, instance=None):
        # TODO: should also support get calendar from dataview
        # self._calendar = None

        self.universe = []
        self._data_api = data_api
        self._trade_api = trade_api
        self._gateway = gateway
        
        self._dataview = dataview
        
        self.instance = instance

        self.strategy = strategy
        self.pm = pm
        
        self.trade_date = 0
        self.time = 0
        self.snapshot = None
        
        self.storage = dict()
        
        for member, obj in self.__dict__.viewitems():
            if hasattr(obj, 'ctx'):
                if member in ['calendar', '_data_api', '_trade_api',
                              '_dataview', '_gateway', 'instance',
                              'pm', 'strategy']:
                    obj.ctx = self

    def save_store(self, path):
        jutil.save_pickle(self.storage, path)

    def load_store(self, path):
        s = jutil.load_pickle(path)
        if s is not None:
            self.storage = s
            
    '''
    @property
    def calendar(self):
        from jaqs.data import Calendar
        if self.data_api is not None:
            return self.data_api.calendar
        elif self._calendar is not None:
            return self._calendar
        else:
            self._calendar = Calendar()
            return self._calendar
        
    '''
    @property
    def data_api(self):
        return self._data_api

    @data_api.setter
    def data_api(self, value):
        if hasattr(value, 'register_context'):
            value.register_context(self)
        self._data_api = value

    @property
    def trade_api(self):
        return self._trade_api

    @trade_api.setter
    def trade_api(self, value):
        if hasattr(value, 'ctx'):
            value.ctx = self
        self._trade_api = value
    
    @property
    def gateway(self):
        return self._gateway

    @gateway.setter
    def gateway(self, value):
        if hasattr(value, 'register_context'):
            value.register_context(self)
        self._gateway = value

    @property
    def dataview(self):
        return self._dataview
    
    @dataview.setter
    def dataview(self, value):
        if hasattr(value, 'register_context'):
            value.register_context(self)
        self._dataview = value
        
    def init_universe(self, univ):
        """
        univ could be single symbol or securities separated by ,
        
        Parameters
        ----------
        univ : str or list
        
        """
        if isinstance(univ, list):
            self.universe = univ
        elif isinstance(univ, (str, unicode)):
            l = univ.split(',')
            l = [x for x in l if x]
            self.universe = l
        else:
            raise NotImplementedError("type of univ is {}".format(type(univ)))


class AlphaContext(Context):
    """
    Attributes
    ----------
    snapshot_sub : pd.DataFrame
        Current snapshot of the universe available to be traded.
        
    """
    def __init__(self, data_api=None, trade_api=None, gateway=None,
                 dataview=None,
                 strategy=None, pm=None, instance=None):
        super(AlphaContext, self).__init__(data_api=data_api, trade_api=trade_api, gateway=gateway,
                                           dataview=dataview,
                                           strategy=strategy, pm=pm, instance=instance)
        self.snapshot_sub = None
    
    
class FuncRegisterable(object):
    """
    A base class for all models. Support function register and context.
    
    Attributes
    ----------
    func_table : dict of {str: RegisteredFunction}
    context : Context
    
    """
    def __init__(self, context=None):
        super(FuncRegisterable, self).__init__()
        self.ctx = context
        self.func_table = dict()
        self.active_funcs = []
    
    def _register_func(self, name, func, options=None):
        if options is None:
            options = dict()
        
        rf = RegisteredFunction(func, name, options)
        self.func_table[name] = rf
        
        self.active_funcs.append(name)
    
    def register_context(self, context):
        self.ctx = context
    '''
    def activate_func(self, f_dict):
        """
        Activate key in self.func_table using kwargs in value.
        
        Parameters
        ----------
        f_dict : dict
            {str: dict}

        """
        self.active_funcs = []
        for f_name, options in f_dict.viewitems():
            self.func_table[f_name].options = options
            self.active_funcs.append(f_name)
    
    '''


class StockSelector(FuncRegisterable):
    def __init__(self, context=None):
        super(StockSelector, self).__init__(context=context)
        pass

    def add_filter(self, name, func, options=None):
        self._register_func(name, func, options)

    def get_selection(self):
        """
        Return a list of stocks that are not selected.
        
        Returns
        -------
        selected : list

        """
        mask_selected = dict()
        for factor in self.active_funcs:
            rf = self.func_table[factor]
            res = rf.func(context=self.ctx, user_options=rf.options)
            res = convert_to_df(res)
            mask_selected[factor] = res
        
        merge = pd.concat(mask_selected.values(), axis=1).astype(float).astype(bool).fillna(False)
        symbol_arr = merge.index.values
        mask_arr = np.all(merge.values, axis=1)
        selected = symbol_arr[mask_arr].tolist()
        return selected


class BaseSignalModel(FuncRegisterable):
    def __init__(self, context=None):
        super(BaseSignalModel, self).__init__(context=context)
        pass

    def add_signal(self, name, func, options=None):
        self._register_func(name, func, options)

    def forecast_signal(self, weights):
        pass


'''
class FactorSignalModel(BaseSignalModel):
    """
    Forecast profit of target weight (portfolio), where:
    
    total_forecast = sum(individual_forecast)
    individual_forecast = combine(individual_forecast from different source)
    
    output: dict
    
    """
    def __init__(self):
        super(FactorSignalModel, self).__init__()
        
        self.total_forecast = None
    
    @staticmethod
    def order2z(order_arr):
        """Transform an array with order to z_score"""
        mean, std = np.mean(order_arr), np.std(order_arr)
        res = (np.asarray(order_arr, dtype=float) - mean) / std
        return res
    
    def forecast_individual(self, symbol):
        forecasts = dict()
        for factor in self.active_funcs:
            rf = self.func_table[factor]
            forecasts[factor] = rf.func(symbol, context=self.context, user_options=rf.options)
        return forecasts
    
    def combine_using_corr(self, forecasts):
        """
        Combine forecasts into one single forecast.
        
        Parameters
        ----------
        forecasts : dict

        Returns
        -------
        res : float

        """
        n = len(forecasts)
        forecast_corr = np.random.randn(n, n)
        forecasts_arr = np.asarray(forecasts.values(), dtype=float).reshape(-1, 1)
        return np.dot(forecast_corr, forecasts_arr).sum()
    
    def combine_sum(self, forecasts):
        res = np.sum(forecasts.values())
        return res
    
    def combine_custom_weight(self, forecasts, forecast_weights):
        res = 0.0
        for factor, f in forecasts.viewitems():
            w = forecast_weights[factor]
            res += f * w
        return res
    
    def forecast_signal(self, weights):
        """
        Forecast total signal of the portfolio with weights.
        
        Parameters
        ----------
        weights : dict

        Returns
        -------
        res : float

        """
        total_signal = 0.0
        for sec, w in weights.viewitems():
            forecasts = self.forecast_individual(sec)
            forecast = self.combine_sum(forecasts)
            
            total_signal += w * forecast
        
        return total_signal

'''


class FactorSignalModel(BaseSignalModel):
    """
    Forecast profit of target weight (portfolio), where:
    
    total_forecast = sum(individual_forecast)
    individual_forecast = combine(individual_forecast from different source)
    
    output: dict
    
    """
    def __init__(self, context=None):
        super(FactorSignalModel, self).__init__(context=context)
    
        self.total_forecast = None

    @staticmethod
    def order2z(order_arr):
        """Transform an array with order to z_score"""
        mean, std = np.mean(order_arr), np.std(order_arr)
        res = (np.asarray(order_arr, dtype=float) - mean) / std
        return res

    def combine_using_corr(self, forecasts):
        """
        Combine forecasts into one single forecast.
        
        Parameters
        ----------
        forecasts : dict

        Returns
        -------
        res : float

        """
        n = len(forecasts)
        forecast_corr = np.random.randn(n, n)
        forecasts_arr = np.asarray(forecasts.values(), dtype=float).reshape(-1, 1)
        return np.dot(forecast_corr, forecasts_arr).sum()

    def get_forecasts(self):
        """
        
        Returns
        -------
        forecasts : dict
            {str: pd.DataFrame}
            DataFrame index is symbol, column is field.

        """
        forecasts = dict()
        for factor in self.active_funcs:
            rf = self.func_table[factor]
            res = rf.func(context=self.ctx, user_options=rf.options)
            res = convert_to_df(res)
            forecasts[factor] = res
        return forecasts
    
    def combine_sum(self, forecasts):
        """
        
        Parameters
        ----------
        forecasts : dict
            {str: pd.DataFrame}
            DataFrame index is symbol, column is field.

        Returns
        -------
        res : dict

        """
        merge = pd.concat(forecasts.values(), axis=1)
        res = merge.sum(axis=1)
        res = res.to_dict()  # convert Series to {label -> value} dict
        return res
    
    def make_forecast(self):
        """
        Get and combine signals (forecasts). Return the combined signal.
        
        Returns
        -------
        forecast : dict of {str: float}
            {symbol_name: forecast_value}
        
        """
        forecasts = self.get_forecasts()  # {str: pd.DataFrame}
        # TODO NaN
        forecasts = {key: value.fillna(0) for key, value in forecasts.viewitems()}
        forecast = self.combine_sum(forecasts)
        return forecast
        
    def forecast_signal(self, weights):
        """
        Forecast total signal of the portfolio with weights.
        
        Parameters
        ----------
        weights : dict

        Returns
        -------
        res : float

        """
        total_signal = 0.0
        forecast_dic = self.make_forecast()
        
        weighted_signal = {key: value * forecast_dic[key] for key, value in weights.viewitems()}
        total_signal = np.sum(weighted_signal.values())
        
        return total_signal


class FactorSignalModel_custom(FactorSignalModel):
    """
    Custom weight.

    """

    def __init__(self, context=None, signal_weights=None):
        super(FactorSignalModel_custom, self).__init__(context=context)

        self.signal_weights = signal_weights

    def combine_custom_weight(self, forecasts):
        td = self.ctx.trade_date

        res = 0.0
        for factor_name, factor_value in forecasts.viewitems():
            weights_dic = self.signal_weights.loc[td, :].to_dict()

            weight = weights_dic[factor_name]
            res += factor_value * weight
        return res

    def make_forecast(self):
        """
        Get and combine signals (forecasts). Return the combined signal.

        Returns
        -------
        forecast : dict of {str: float}
            {symbol_name: forecast_value}

        """
        forecasts = self.get_forecasts()  # {str: pd.DataFrame}
        # TODO NaN
        forecasts = {key: value.fillna(0) for key, value in forecasts.viewitems()}
        forecast = self.combine_custom_weight(forecasts)
        return forecast


class BaseCostModel(FuncRegisterable):
    """Transaction Cost = commission + spread + execution"""
    def __init__(self, context=None):
        super(BaseCostModel, self).__init__(context=context)
        pass
    
    def consider_cost(self, name, func, options=None):
        self._register_func(name, func, options=options)
    
    def add_cost(self, name, func, options=None):
        self._register_func(name, func, options=options)


class SimpleCostModel(BaseCostModel):
    def __init__(self, context=None):
        super(SimpleCostModel, self).__init__(context=context)
    
    def calc_individual_cost(self, symbol, turnover):
        # following data are fetched from the data server
        avg_bid_ask_spread = 1.0
        avg_daily_volume = 1e7
        yearly_volatility = 0.2
        price = 1e3
        trading_volume = turnover / price
        
        limit_reached = False
        if limit_reached:
            commission_rate = 1e9
        else:
            commission_rate = 1e-4
        
        cost_default = self._calc_individual_cost(trading_volume, commission_rate, price,
                                                  avg_bid_ask_spread, avg_daily_volume, yearly_volatility)

        cost_user_dic = dict()
        for cost_name in self.active_funcs:
            rf = self.func_table[cost_name]
            cost_user_dic[cost_name] = rf.func(symbol, trading_volume * price,
                                               context=self.ctx, user_options=rf.options)
        cost_user = sum(cost_user_dic.values())
        
        return cost_default + cost_user
    
    @staticmethod
    def _calc_individual_cost(size, price, rate, avg_ba, adv, vol):
        """
        This function serves as a mathematical formula.
        
        Parameters
        ----------
        size : float
            Trading volume.
        rate : float
            Rate of commission.
        price : float
            Current price of the symbol.
        avg_ba : float
            Average bid-ask spread.
        adv : float
            Average daily trading volume.
        vol : float
            Volatility.

        Returns
        -------
        res : float

        """
        commission = rate * size * price
        
        spread_cost = avg_ba * 0.5
        
        kappa = 1e-6
        gamma = 1.5
        execution_cost = kappa * np.power(size, gamma)
        
        cost = commission + spread_cost + execution_cost  # 50bp
        
        return cost
    
    def calc_cost(self, weights_last, weights_now):
        """
        Calculate transaction cost from current position to target position.
        
        Parameters
        ----------
        weights_last : dict
        weights_now : dict
            Target positions.

        Returns
        -------
        total_cost : float

        """
        total_cost = 0.0
        for sec, w_last in weights_last.viewitems():
            w_now = weights_now[sec]
            
            trading_turnover = abs(w_now - w_last)
            total_cost += self.calc_individual_cost(sec, trading_turnover)
        
        return total_cost


class BaseRiskModel(FuncRegisterable):
    def __init__(self, context=None):
        super(BaseRiskModel, self).__init__(context=context)
        pass
    
    def consider_risk(self, name, func, options=None):
        self._register_func(name, func, options=options)


class FactorRiskModel(BaseRiskModel):
    def __init__(self, context=None):
        super(FactorRiskModel, self).__init__(context=context)
        
        self.benchmark = ""
    
    def set_benchmark(self, benchmark):
        self.benchmark = benchmark
    
    def calc_risk(self, weights):
        """
        Calculate total risk of portfolio.
        
        Parameters
        ----------
        weights : dict
            {int: str}

        Returns
        -------
        total_risk : float

        """
        idiosyncratic_risk = self.calc_idiosyncratic_risk(weights)
        n = len(weights)
        corr_mat = np.random.randn(n, n)
        weights_arr = np.asarray(weights.values(), dtype=float)
        factor_risk = weights_arr.dot(corr_mat).dot(weights_arr)
        total_risk = factor_risk + idiosyncratic_risk
        return total_risk
    
    """
    def calc_factor_risk(weight):
        risk_factors = [10000.0, -20000.0]
        risk_factor = 100000.0  # beta risk
        return risk_factors, risk_factor
    """
    
    def _get_idiosyncratic_risk(self, sec):
        return 0.0
    
    def calc_idiosyncratic_risk(self, weights):
        """Calculate weighted sum of idiosyncratic risks of all securities."""
        res = 0.0
        for sec, w in weights.viewitems():
            res += w * self._get_idiosyncratic_risk(sec)
        return res


def test_models():
    weight_last = {'symbol1': 0.2, 'symbolB': 0.8}
    weight_now = {'symbol1': 0.3, 'symbolB': 0.7}
    
    portfolio = 1e7
    weight_last = {k: v * portfolio for k, v in weight_last.viewitems()}
    weight_now = {k: v * portfolio for k, v in weight_now.viewitems()}
    
    signal = FactorSignalModel().forecast_signal(weight_now)
    cost = SimpleCostModel().calc_cost(weight_last, weight_now)
    # liquid = Liquidation().calc_liquid(weight_now)
    risk = FactorRiskModel().calc_risk(weight_now)
    
    risk_coef = 1.0
    cost_coef = 1.0
    util = signal - risk_coef * risk - cost_coef * cost  # - liquid * liq_factor
    
    """
    obj.: h(x) = argmax(util)
    s.t.: g1(x) = abs(risk_factors[3]) < 100000.0
    g2(x) = var(risk_factors) <= 0.0
    """


def convert_to_df(res):
    if isinstance(res, pd.DataFrame):
        pass
    elif isinstance(res, pd.Series):
        res = pd.DataFrame(index=res.index, data=res.values)
    else:
        raise ValueError("Return type of signal function must be DataFrame or Series or dict!"
                         + "\nWe got [{}] instead.".format(type(res)))
    '''
    elif isinstance(res, dict):
        res = pd.DataFrame(columns=[factor], data=pd.Series(data=res))
    '''
    return res

    
if __name__ == "__main__":
    test_models()



from collections import namedtuple
import datetime  as dt
import pandas    as pd
import numpy     as np

long_nan = 9223372036854775807

def is_long_nan(v):
    if v == long_nan:
        return True
    else :
        return False
    
def to_nan(x):
    if is_long_nan(x):    
        return np.nan
    else:
        return x
        
def _to_date(row):
    date = int(row['DATE'])
    return pd.datetime( year=date/10000, month = date/ 100 % 100, day = date%100)

def _to_datetime(row):
    date = int(row['DATE'])
    time = int(row['TIME']) / 1000
    return pd.datetime( year=date/10000, month=date/ 100 % 100, day = date%100,
                        hour=time/10000, minute=time/100%100, second =time%100)

def _to_dataframe(cloumset, index_func = None, index_column = None):
    df = pd.DataFrame(cloumset)
    for col in df.columns:
        if df.dtypes.loc[col] == np.int64:
            df.loc[:,col] = df.loc[:,col].apply(to_nan)            
    if index_func:
        df.index = df.apply(index_func, axis = 1)
    elif index_column:
        df.index = df[index_column]
        del df.index.name

    return df

def _error_to_str(error):
    if error:
        if error.has_key('message'):
            return str(error['error']) + "," + error['message']
        else:
            return str(error['error']) + ","
    else:
        return ","


def to_obj(class_name, data):
    try:
        if type(data) == list or type(data) == tuple:
            result = []
            for d in data:
                result.append( namedtuple(class_name, d.keys())(*d.values()) )
            return result

        elif type(data) == dict :
            result = namedtuple(class_name, data.keys())(*data.values())
            return result
        else:
            return data
    except Exception, e:
        print class_name, data, e
        return data

def to_date_int(date):
    if isinstance(date, (str, unicode)):
        t = dt.datetime.strptime(date, "%Y-%m-%d")
        date_int = t.year * 10000 + t.month * 100 + t.day
        return date_int
    elif isinstance(date, (int, np.integer)):
        return date
    else:
        return -1

def to_time_int(time):
    if isinstance(time, (str, unicode)):
        t = dt.datetime.strptime(time, "%H:%M:%S")
        time_int = t.hour * 10000 + t.minute * 100 + t.second
        return time_int
    elif isinstance(time, (int, np.integer)):
        return time
    else:
        return -1 

def extract_result(cr, data_format="", index_column=None, class_name=""):
    """
        format supports pandas, obj.
    """
    
    err = _error_to_str(cr['error']) if cr.has_key('error') else None
    if cr.has_key('result'):
        if data_format == "pandas":
            if index_column :
                return (_to_dataframe(cr['result'], None, index_column), err)
            # if 'TIME' in cr['result']:
            #     return (_to_dataframe(cr['result'], _to_datetime), err)
            # elif 'DATE' in cr['result']:
            #     return (_to_dataframe(cr['result'], _to_date), err)
            else:
                return (_to_dataframe(cr['result']), err)

        elif data_format == "obj" and cr['result'] and class_name:
            r = cr['result']
            if type(r) == list or type(r) == tuple:
                result = []
                for d in r:
                    result.append( namedtuple(class_name, d.keys())(*d.values()) )
            elif type(r) == dict :
                result = namedtuple(class_name, r.keys())(*r.values())
            else:
                result = r

            return (result, err)
        else:            
            return (cr['result'], err)
    else:
        return (None, err)



# encoding: utf-8

import jaqs.util as jutil


def test_read_save_pickle():
    fp = '../../output/tests/test_read_save_pickle.pic'
    d = {'a': 1.0, 'b': 2, 'c': True, 'd': list()}
    jutil.save_pickle(d, fp)
    
    d2 = jutil.load_pickle(fp)
    assert d2['b'] == 2
    
    d3 = jutil.load_pickle('a_non_exits_file_blabla.pic')
    assert d3 is None


if __name__ == "__main__":
    import time
    t_start = time.time()
    
    g = globals()
    g = {k: v for k, v in g.viewitems() if k.startswith('test_') and callable(v)}
    
    for test_name, test_func in g.viewitems():
        print "\nTesting {:s}...".format(test_name)
        test_func()
    print "Test Complete."
    
    t3 = time.time() - t_start
    print "\n\n\nTime lapsed in total: {:.1f}".format(t3)



# encoding: utf-8

import os
import numpy as np
import pandas as pd

from . import performance as pfm
from . import plotting

import jaqs.util as jutil


class SignalDigger(object):
    """
    
    Attributes
    ----------
    signal_data : pd.DataFrame - MultiIndex
        Index is pd.MultiIndex ['trade_date', 'symbol'], columns = ['signal', 'return', 'quantile']
    period : int
        Horizon used to calculate return.
    n_quantiles : int
    output_format : str
    output_folder : str
        
    """
    def __init__(self, output_folder=".", output_format='pdf'):
        self.output_format = output_format
        self.output_folder = os.path.abspath(output_folder)
        
        self.signal_data = None
        self.period = None
        self.n_quantiles = 5
        self.benchmark_ret = None
        
        self.returns_report_data = dict()
        self.ic_report_data = dict()
        self.fig_data = dict()
        self.fig_objs = dict()

    def process_signal_before_analysis(self,
                                       signal, price=None, ret=None, benchmark_price=None,
                                       period=5, n_quantiles=5,
                                       mask=None):
        """
        Prepare for signal analysis.

        Parameters
        ----------
        signal : pd.DataFrame
            Index is date, columns are stocks.
        price : pd.DataFrame
            Index is date, columns are stocks.
        ret : pd.DataFrame
            Index is date, columns are stocks.
        benchmark_price : pd.DataFrame or pd.Series
            Price of benchmark.
        mask : pd.DataFrame
            Data cells that should NOT be used.
        n_quantiles : int
        period : int
            periods to compute forward returns on.

        Returns
        -------
        res : pd.DataFrame
            Index is pd.MultiIndex ['trade_date', 'symbol'], columns = ['signal', 'return', 'quantile']
            
        """
        """
        Deal with suspensions:
            If the period of calculating return is d (from T to T+d), then
            we do not use signal values of those suspended on T,
            we do not calculate return for those suspended on T+d.
        """
        # ----------------------------------------------------------------------
        # parameter validation
        if price is None and ret is None:
            raise ValueError("One of price / ret must be provided.")
        if price is not None and ret is not None:
            raise ValueError("Only one of price / ret should be provided.")
        if ret is not None and benchmark_price is not None:
            raise ValueError("You choose 'return' mode but benchmark_price is given.")
        
        data = price if price is not None else ret
        assert np.all(signal.index == data.index)
        assert np.all(signal.columns == data.columns)
        if mask is not None:
            assert np.all(signal.index == mask.index)
            assert np.all(signal.columns == mask.columns)
            mask = jutil.fillinf(mask)
            mask = mask.astype(int).fillna(0).astype(bool)
        else:
            mask = pd.DataFrame(index=signal.index, columns=signal.columns, data=False)
        signal = jutil.fillinf(signal)
        data = jutil.fillinf(data)

        # ----------------------------------------------------------------------
        # save data
        self.n_quantiles = n_quantiles
        self.period = period

        # ----------------------------------------------------------------------
        # fwd_returns are processed forward returns
        signal = signal.shift(self.period)
        
        if price is not None:
            df_return = pfm.price2ret(price, self.period)
            if benchmark_price is not None:
                benchmark_price = benchmark_price.loc[signal.index]
                bench_ret = pfm.price2ret(benchmark_price, self.period, axis=0)
                self.benchmark_ret = bench_ret
                df_return = df_return.sub(bench_ret.values.flatten(), axis=0)
        else:
            df_return = ret

        # ----------------------------------------------------------------------
        # get masks
        mask_prices = data.isnull()
        # Because we use FORWARD return, if one day's price is broken, the day that is <period> days ago is also broken.
        mask_prices = np.logical_or(mask_prices, mask_prices.shift(self.period).fillna(True))
        mask_signal = signal.isnull()

        mask = np.logical_or(mask, mask_prices)
        mask = np.logical_or(mask, mask_signal)

        if price is not None:
            mask_forward = np.logical_or(mask, mask.shift(self.period).fillna(True))
            mask = np.logical_or(mask, mask_forward)

        # ----------------------------------------------------------------------
        # calculate quantile
        signal_masked = signal.copy()
        signal_masked = signal_masked[~mask]
        df_quantile = jutil.to_quantile(signal_masked, n_quantiles=n_quantiles)

        # ----------------------------------------------------------------------
        # stack
        def stack_td_symbol(df):
            df = pd.DataFrame(df.stack(dropna=False))  # do not dropna
            df.index.names = ['trade_date', 'symbol']
            df.sort_index(axis=0, level=['trade_date', 'symbol'], inplace=True)
            return df
        
        mask = stack_td_symbol(mask)
        df_quantile = stack_td_symbol(df_quantile)
        df_return = stack_td_symbol(df_return)

        # ----------------------------------------------------------------------
        # concat signal value
        res = stack_td_symbol(signal)
        res.columns = ['signal']
        res['return'] = df_return
        res['quantile'] = df_quantile
        res = res.loc[~(mask.iloc[:, 0]), :]
        
        print "Nan Data Count (should be zero) : {:d};  " \
              "Percentage of effective data: {:.0f}%".format(res.isnull().sum(axis=0).sum(),
                                                             len(res) * 100. / signal.size)
        res = res.astype({'signal': float, 'return': float, 'quantile': int})
        self.signal_data = res
    
    def show_fig(self, fig, file_name):
        """
        Save fig object to self.output_folder/filename.
        
        Parameters
        ----------
        fig : matplotlib.figure.Figure
        file_name : str

        """
        
        self.fig_objs[file_name] = fig
        
        if self.output_format in ['pdf', 'png', 'jpg']:
            fp = os.path.join(self.output_folder, '.'.join([file_name, self.output_format]))
            jutil.create_dir(fp)
            fig.savefig(fp)
            print "Figure saved: {}".format(fp)
        elif self.output_format == 'base64':
            fig_b64 = jutil.fig2base64(fig, 'png')
            self.fig_data[file_name] = fig_b64
            print "Base64 data of figure {} will be stored in dictionary.".format(file_name)
        elif self.output_format == 'plot':
            fig.show()
        else:
            raise NotImplementedError("output_format = {}".format(self.output_format))
    
    @plotting.customize
    def create_returns_report(self):
        """
        Creates a tear sheet for returns analysis of a signal.

        """
        n_quantiles = self.signal_data['quantile'].max()
        
        # ----------------------------------------------------------------------------------
        # Daily Signal Return Time Series
        # Use regression or weighted average to calculate.
        period_wise_long_ret =\
            pfm.calc_period_wise_weighted_signal_return(self.signal_data, weight_method='long_only')
        period_wise_short_ret = \
            pfm.calc_period_wise_weighted_signal_return(self.signal_data, weight_method='short_only')
        cum_long_ret = pfm.period_wise_ret_to_cum(period_wise_long_ret, period=self.period, compound=False)
        cum_short_ret = pfm.period_wise_ret_to_cum(period_wise_short_ret, period=self.period, compound=False)
        # period_wise_ret_by_regression = perf.regress_period_wise_signal_return(signal_data)
        # period_wise_ls_signal_ret = \
        #     pfm.calc_period_wise_weighted_signal_return(signal_data, weight_method='long_short')
        # daily_ls_signal_ret = pfm.period2daily(period_wise_ls_signal_ret, period=period)
        # ls_signal_ret_cum = pfm.daily_ret_to_cum(daily_ls_signal_ret)

        # ----------------------------------------------------------------------------------
        # Period-wise Quantile Return Time Series
        # We calculate quantile return using equal weight or market value weight.
        # Quantile is already obtained according to signal values.
        
        # quantile return
        period_wise_quantile_ret_stats = pfm.calc_quantile_return_mean_std(self.signal_data, time_series=True)
        cum_quantile_ret = pd.concat({k: pfm.period_wise_ret_to_cum(v['mean'], period=self.period, compound=False)
                                      for k, v in period_wise_quantile_ret_stats.items()},
                                     axis=1)
        
        # top quantile minus bottom quantile return
        period_wise_tmb_ret = pfm.calc_return_diff_mean_std(period_wise_quantile_ret_stats[n_quantiles],
                                                            period_wise_quantile_ret_stats[1])
        cum_tmb_ret = pfm.period_wise_ret_to_cum(period_wise_tmb_ret['mean_diff'], period=self.period, compound=False)

        # ----------------------------------------------------------------------------------
        # Alpha and Beta
        # Calculate using regression.
        '''
        weighted_portfolio_alpha_beta
        tmb_alpha_beta =
        '''
        
        # start plotting
        if self.output_format:
            vertical_sections = 6
            gf = plotting.GridFigure(rows=vertical_sections, cols=1)
            gf.fig.suptitle("Returns Tear Sheet\n\n(no compound)\n (period length = {:d} days)".format(self.period))
    
            plotting.plot_quantile_returns_ts(period_wise_quantile_ret_stats,
                                              ax=gf.next_row())

            plotting.plot_cumulative_returns_by_quantile(cum_quantile_ret,
                                                         ax=gf.next_row())

            plotting.plot_cumulative_return(cum_long_ret,
                                            title="Signal Weighted Long Only Portfolio Cumulative Return",
                                            ax=gf.next_row())
            
            plotting.plot_cumulative_return(cum_short_ret,
                                            title="Signal Weighted Short Only Portfolio Cumulative Return",
                                            ax=gf.next_row())

            plotting.plot_mean_quantile_returns_spread_time_series(period_wise_tmb_ret, self.period,
                                                                   bandwidth=0.5,
                                                                   ax=gf.next_row())
            
            plotting.plot_cumulative_return(cum_tmb_ret,
                                            title="Top Minus Bottom (long top, short bottom)"
                                                  "Portfolio Cumulative Return",
                                            ax=gf.next_row())

            self.show_fig(gf.fig, 'returns_report')
        
        self.returns_report_data = {'period_wise_quantile_ret': period_wise_quantile_ret_stats,
                                    'cum_quantile_ret': cum_quantile_ret,
                                    'cum_long_ret': cum_long_ret,
                                    'cum_short_ret': cum_short_ret,
                                    'period_wise_tmb_ret': period_wise_tmb_ret,
                                    'cum_tmb_ret': cum_tmb_ret}

    @plotting.customize
    def create_information_report(self):
        """
        Creates a tear sheet for information analysis of a signal.
        
        """
        ic = pfm.calc_signal_ic(self.signal_data)
        ic.index = pd.to_datetime(ic.index, format="%Y%m%d")
        monthly_ic = pfm.mean_information_coefficient(ic, "M")

        if self.output_format:
            ic_summary_table = pfm.calc_ic_stats_table(ic)
            plotting.plot_information_table(ic_summary_table)
            
            columns_wide = 2
            fr_cols = len(ic.columns)
            rows_when_wide = (((fr_cols - 1) // columns_wide) + 1)
            vertical_sections = fr_cols + 3 * rows_when_wide + 2 * fr_cols
            gf = plotting.GridFigure(rows=vertical_sections, cols=columns_wide)
            gf.fig.suptitle("Information Coefficient Report\n\n(period length = {:d} days)"
                            "\ndaily IC = rank_corr(period-wise forward return, signal value)".format(self.period))

            plotting.plot_ic_ts(ic, self.period, ax=gf.next_row())
            plotting.plot_ic_hist(ic, self.period, ax=gf.next_row())
            # plotting.plot_ic_qq(ic, ax=ax_ic_hqq[1::2])

            plotting.plot_monthly_ic_heatmap(monthly_ic, period=self.period, ax=gf.next_row())
        
            self.show_fig(gf.fig, 'information_report')
        
        self.ic_report_data = {'daily_ic': ic,
                               'monthly_ic': monthly_ic}

    @plotting.customize
    def create_full_report(self):
        """
        Creates a full tear sheet for analysis and evaluating single
        return predicting (alpha) signal.
        
        """
        # signal quantile description statistics
        qstb = calc_quantile_stats_table(self.signal_data)
        if self.output_format:
            plotting.plot_quantile_statistics_table(qstb)
            
        self.create_returns_report()
        self.create_information_report()
        # we do not do turnover analysis for now
        # self.create_turnover_report(signal_data)
        
        res = dict()
        res.update(self.returns_report_data)
        res.update(self.ic_report_data)
        res.update(self.fig_data)
        return res


def calc_quantile_stats_table(signal_data):
    quantile_stats = signal_data.groupby('quantile').agg(['min', 'max', 'mean', 'std', 'count'])['signal']
    quantile_stats['count %'] = quantile_stats['count'] / quantile_stats['count'].sum() * 100.
    return quantile_stats



# encoding: utf-8

import os
import codecs

import jinja2

import jaqs.util as jutil
# from weasyprint import HTML


class Report(object):
    def __init__(self, dic, source_dir, template_fn, out_folder='.'):
        """

        Parameters
        ----------
        dic : dict
        source_dir : str
            path of directory where HTML template and css files are stored.
        template_fn : str
            File name of HTML template.
        out_folder : str
            Output folder of report.

        """
        self.dic = dic
        self.template_fn = template_fn
        self.out_folder = os.path.abspath(out_folder)

        self.env = jinja2.Environment(loader=jinja2.FileSystemLoader(searchpath=[source_dir]))
        self._update_env()

        self.template = self.env.get_template(self.template_fn)
        self.html = ""

    def _update_env(self):
        """Define custom functions we use in HTML template."""
        def round_if_float(x, n):
            if isinstance(x, float):
                return round(x, n)
            else:
                return x
        self.env.filters.update({'round_if_float': round_if_float})
    
    def generate_html(self):
        self.html = self.template.render(self.dic)
    
    def output_html(self, fn='test_out.html'):
        path = os.path.abspath(os.path.join(self.out_folder, fn))
        
        jutil.create_dir(path)
        with codecs.open(path, 'w', encoding='utf-8') as f:
            f.write(self.html)

        print "HTML report: {:s}".format(path)
    
    def output_pdf(self, fn='test_out.html'):
        pass
        # h = HTML(string=self.html)
        # h.write_pdf(os.path.join(self.out_folder, fn))#, stylesheets=[self.fp_css])



# encoding: utf-8

import datetime as dt

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

from jaqs.data.basic import InstManager
from jaqs.trade import common
from jaqs.data.basic import Trade
# from jaqs.data import Calendar


# %matplotlib inline


class Pnl(object):
    def __init__(self):
        self.date = 0
        self.symbol = ''
        self.hold_pnl = 0.0
        self.trade_pnl = 0.0


class PnlReport:
    def __init__(self):
        self.daily_pnls = []
        self.trades = None
        self.hold_pnl = 0.0
        self.trade_pnl = 0.0
        self.trade_amount = 0.0
        self.trade_count = 0.0
        self.total_pnl = 0.0
        self.tax = 0.0
        self.commission = 0.0
        self.win_rate = 0.0
        self.maxdrawdown = 0.0
        self.sharp = 0.0


class DailyPnlReport(object):
    def __init__(self):
        self.positions = {}
        self.total_pnl = 0.0
        self.hold_pnl = 0.0
        self.trade_pnl = 0.0
        self.accum_total_pnl = 0.0
        self.trade_amount = 0.0
        self.trade_count = 0
        self.commission = 0.0
        self.tax = 0.0
        self.date = 0
    
    def copy(self, pnl):
        self.positions = pnl.positions
        self.total_pnl = pnl.total_pnl
        self.hold_pnl = pnl.hold_pnl
        self.trade_pnl = pnl.trade_pnl
        self.trade_amount = pnl.trade_amount
        self.trade_count = pnl.trade_count
        self.commission = pnl.commission
        self.tax = pnl.tax
        self.date = pnl.date


class PnlManager(object):
    def __init__(self):
        self.calendar = Calendar()
        self.instmgr = InstManager()
        self.strategy = None
        self.pnls = []
        self.start_date = 0
        self.end_date = 0
        self.close_prices = {}
        self.universe = []
        
        self.data_api = None
    
    def setStrategy(self, sta):
        self.strategy = sta
    
    def tradeToDataframe(self, trades):
        
        key_list = ['fill_no', 'entrust_no', 'symbol', 'entrust_action', 'fill_price', 'fill_size', 'fill_date',
                    'fill_time']
        
        result = {}
        
        for p in trades:
            pos = {}
            for key in key_list:
                if key == 'fill_no':
                    pos[key] = int(p.__getattribute__(key))
                else:
                    pos[key] = p.__getattribute__(key)
            result[int(p.fill_no)] = pos
        
        return pd.DataFrame(result).transpose().sort_values(['fill_date', 'fill_time'])
    
    def generateStatisticReport(self, pnls):
        report = PnlReport()
        report.daily_pnls = pnls
        report.trades = self.tradeToDataframe(self.ctx.pm.trades)
        win_count = 0
        i = 0
        pre_value = self.strategy.init_balance
        cur_value = self.strategy.init_balance
        rtn = []
        
        for pnl in pnls:
            cur_value = pre_value + pnl.total_pnl
            rtn.append(cur_value / pre_value - 1.0)
            pre_value = cur_value
            report.trade_amount += pnl.trade_amount
            report.trade_count += pnl.trade_count
            report.commission += pnl.commission
            report.tax += pnl.tax
            report.total_pnl += pnl.total_pnl
            pnl.accum_total_pnl = report.total_pnl
            if report.maxdrawdown > pnl.accum_total_pnl:
                report.maxdrawdown = pnl.accum_total_pnl
            if pnl.total_pnl > 0.0:
                win_count += 1
        
        date_num = float(len(pnls))
        report.win_rate = win_count / date_num
        s = pd.Series(rtn)
        mean_value = s.mean()
        std_value = s.std()
        report.sharp = mean_value / std_value * np.sqrt(240.0)
        return report
    
    def generateReport(self, output_format=""):
        daily_pnls = self.calcPnl(self.ctx.pm.trades)
        report = self.generateStatisticReport(daily_pnls)
        print "Total PNL: %f" % (report.total_pnl)
        print "Total trade number: %d" % (report.trade_count)
        print "Total trade amount: %f" % (report.trade_amount)
        print "Win rate: %f" % (report.win_rate)
        print "Max drawdown: %f" % (report.maxdrawdown)
        print "Tax: %f" % (report.tax)
        print "Commission: %f" % (report.commission)
        print "Sharp: %f" % (report.sharp)
        trade_pnl = []
        hold_pnl = []
        total_pnl = []
        dates = []
        trade_amount = []
        
        for pnl in daily_pnls:
            dates.append(str(pnl.date))
            trade_pnl.append(pnl.trade_pnl)
            hold_pnl.append(pnl.hold_pnl)
            total_pnl.append(pnl.accum_total_pnl)
            trade_amount.append(pnl.trade_amount)
        
        plt.figure(figsize=(19, 12))
        plt.subplot(111)
        plt.title("Strategy Backtest Result")
        plt.ylabel("PNL")
        plt.xlabel("Date")
        xs = [dt.datetime.strptime(d, '%Y%m%d').date() for d in dates]
        plt.plot(xs, trade_pnl, color='r', label='Trading PNL', linewidth="2")
        plt.plot(xs, hold_pnl, color='c', label='Holding PNL', linewidth="2")
        plt.plot(xs, total_pnl, color='m', label='Total PNL', linewidth="2")
        plt.legend(loc='best')
        plt.grid(True, linestyle="--", color="black", linewidth="1")
        #        legend = ax.legend(loc='upper center', shadow=True, fontsize='x-large')
        if output_format == 'plot':
            plt.show()
        return report
    
    def initFromConfig(self, props, data_server):
        self.data_api = data_server
        
        self.start_date = props.get('start_date')
        self.end_date = props.get('end_date')
        self.fut_commission = props.get('future_commission_rate', 0.0)
        self.stk_commission = props.get('stock_commission_rate', 0.0)
        self.stk_tax = props.get('stock_tax_rate', 0.0)
        self.universe = []
        universe_str = props.get('symbol')
        for code in universe_str.split(','):
            self.universe.append(code.strip())
        self.prepareData()
    
    def prepareData(self):
        begindate = self.calendar.get_last_trade_date(self.start_date)
        enddate = self.calendar.get_next_trade_date(self.end_date)
        for symbol in self.universe:
            df, msg = self.data_api.daily(symbol, start_date=begindate, end_date=enddate, fields="",
                                          adjust_mode='post')
            for idx, row in df.iterrows():
                date = row['trade_date']
                close = row['close']
                if not self.close_prices.has_key(date):
                    self.close_prices[date] = {}
                self.close_prices[date][symbol] = close
    
    def isBuyAction(self, action):
        if (action == common.ORDER_ACTION.BUY
            or action == common.ORDER_ACTION.COVER
            or action == common.ORDER_ACTION.COVERYESTERDAY
            or action == common.ORDER_ACTION.COVERTODAY):
            return True
        else:
            return False
    
    def isSellAction(self, action):
        if (action == common.ORDER_ACTION.SELL
            or action == common.ORDER_ACTION.SELLTODAY
            or action == common.ORDER_ACTION.SELLYESTERDAY
            or action == common.ORDER_ACTION.SHORT):
            return True
        else:
            return False
    
    def calculatePnls(self):
        hold_pnls = self.calculateHoldPnl()
        trade_pnls = self.calculateTradePnl()
        total_pnls = []
        i = 0
        for hold_pnl in hold_pnls:
            trade_pnl = trade_pnls[i]
            pnl = DailyPnlReport()
            pnl.date = hold_pnl.date
            pnl.hold_pnl = hold_pnl.hold_pnl
            pnl.trade_amount = trade_pnl.trade_amount
            pnl.trade_pnl = trade_pnl.trade_pnl
            pnl.commission = trade_pnl.commision
            pnl.tax = trade_pnl.tax
            pnl.total_pnl = pnl.hold_pnl + pnl.trade_pnl
            total_pnls.append(pnl)
            if hold_pnl.date == trade_pnl.date:
                i += 1
        return total_pnls
    
    def calculateHoldPnl(self):
        dates = self.calendar.get_trade_date_range(self.start_date, self.end_date)
        hold_pnl = []
        for date in dates:
            hold_pnl.append(self.calcOneDayHoldPnl(date))
        return hold_pnl
    
    def calcOneDayHoldPnl(self, date, position):
        pnl = 0.0
        pre_date = self.calendar.get_last_trade_date(date)
        pre_close_prices = self.close_prices[pre_date]
        cur_close_prices = self.close_prices[date]
        for code, hold_size in position.viewitems():
            # hold pnl calc          
            
            pre_close = pre_close_prices.get(code, 0.0)
            close = cur_close_prices.get(code, 0.0)
            inst = self.instmgr.get_instrument(code)
            mult = inst.multiplier
            hold_pnl = hold_size * mult * (close - pre_close)
            pnl += hold_pnl
        return pnl
    
    def calcPnl(self, all_trades):
        trades = {}
        trade_pnls = {}
        pnls = []
        for trade in all_trades:
            code = trade.symbol
            date = trade.fill_date
            if trades.has_key(date) == False:
                trades[date] = {}
            if trades[date].has_key(code) == False:
                trades[date][code] = []
            trades[date][code].append(trade)
        
        for key, value in trades.viewitems():
            pnl = self.calcOneDayTradePnl(key, value)
            trade_pnls[pnl.date] = pnl
        dates = self.calendar.get_trade_date_range(self.start_date, self.end_date)
        i = 0
        pre_position = {}
        cur_position = {}
        for date in dates:
            pnl = DailyPnlReport()
            pnl.date = date
            tpnl = trade_pnls.get(date, None)
            if tpnl != None:
                pnl.copy(tpnl)
            pnl.positions = self.combinePosition(pre_position, pnl.positions)
            pnl.hold_pnl = self.calcOneDayHoldPnl(date, pre_position)
            pnl.total_pnl = pnl.hold_pnl + pnl.trade_pnl - pnl.tax - pnl.commission
            pre_position = pnl.positions
            pnls.append(pnl)
        self.pnls = pnls
        return pnls
    
    #     def calcTradePnl(self, trade, inst, close):
    #         action = trade.entrust_action
    #         pnl = 0.0
    #         if self.isBuyAction(action):
    #             pnl = (close - trade.fill_price) * trade.fill_size * inst.multiplier
    #         elif self.isSellAction(action):
    #             pnl = (trade.fill_price - close) * trade.fill_size * inst.multiplier
    #         else :
    #             return 0.0
    #         return pnl
    
    # combine src position to dst_position
    def combinePosition(self, cur_position, pre_position):
        position = {}
        for k, v in cur_position.viewitems():
            position[k] = v
        for k, v in pre_position.viewitems():
            if position.has_key(k) == False:
                position[k] = v
            else:
                position[k] += v
        return position
    
    def calcOneDayTradePnl(self, date, trade_map):
        pnl = DailyPnlReport()
        pnl.date = date
        close_prices = self.close_prices[date]
        for code, trades in trade_map.viewitems():
            position = 0
            close = close_prices.get(code, 0.0)
            inst = self.instmgr.get_instrument(code)
            tax_rate = 0.0
            commission_rate = 0.0
            
            if inst.is_stock():
                tax_rate = self.stk_tax
                commission_rate = self.stk_commission
            elif inst.is_future():
                commission_rate = self.fut_commission
            for trade in trades:
                tax = 0.0
                action = trade.entrust_action
                trade_pnl = 0.0
                trade_amount = trade.fill_price * trade.fill_size * inst.multiplier
                commission = commission_rate * trade_amount
                pnl.trade_amount += trade_amount
                pnl.trade_count += 1
                if self.isBuyAction(action):
                    position += trade.fill_size
                    trade_pnl = (close - trade.fill_price) * trade.fill_size * inst.multiplier
                elif self.isSellAction(action):
                    position -= trade.fill_size
                    tax = trade_amount * tax_rate
                    trade_pnl = (trade.fill_price - close) * trade.fill_size * inst.multiplier
                pnl.commission += commission
                pnl.tax += tax
                pnl.trade_pnl += trade_pnl
                pnl.trade_amount += trade_amount
            pnl.positions[code] = position
        
        return pnl


if __name__ == '__main__':
    props = dict()
    props['start_date'] = 20170702
    props['end_date'] = 20170712
    props['future_commission_rate'] = 0.005
    props['stock_commission_rate'] = 0.005
    props['stock_tax_rate'] = 0.002
    props['symbol'] = '600030.SH'
    pnlmgr = PnlManager()
    
    from jaqs.data import RemoteDataService
    ds = RemoteDataService()
    pnlmgr.initFromConfig(props, ds)
    trades = []
    t1 = Trade()
    t1.symbol = '600030.SH'
    t1.action = common.ORDER_ACTION.BUY
    t1.fill_date = 20170704
    t1.fill_size = 100
    t1.fill_price = 16.72
    t2 = Trade()
    t2.symbol = '600030.SH'
    t2.action = common.ORDER_ACTION.SELL
    t2.fill_date = 20170706
    t2.fill_size = 50
    t2.fill_price = 16.69
    t3 = Trade()
    t3.symbol = '600030.SH'
    t3.action = common.ORDER_ACTION.SELL
    t3.fill_date = 20170707
    t3.fill_size = 50
    t3.fill_price = 16.75
    trades.append(t1)
    trades.append(t2)
    trades.append(t3)
    pnls = pnlmgr.calcPnl(trades)
    for pnl in pnls:
        print pnl.date, pnl.trade_pnl, pnl.hold_pnl, pnl.positions.get('600030.SH')



# encoding: UTF-8

from jaqs.trade import common

'''
本文件仅用于存放对于事件类型常量的定义。

由于python中不存在真正的常量概念，因此选择使用全大写的变量名来代替常量。
这里设计的命名规则以EVENT_前缀开头。

常量的内容通常选择一个能够代表真实意义的字符串（便于理解）。

建议将所有的常量定义放在该文件中，便于检查是否存在重复的现象。
'''


class EVENT_TYPE(common.ReprStrEnum):
    TIMER = 'timer'  # 计时器事件，每隔1秒发送一次
    MARKET_DATA = 'market_data'  # 行情事件

    ORDER_RSP = 'order_rsp'
    TASK_RSP = 'task_rsp'
    TASK_STATUS_IND = 'task_callback'
    TRADE_IND = 'trade_ind'  # 成交回报
    ORDER_STATUS_IND = 'order_status_ind'  # 状态回报
    
    PLACE_ORDER = 'place_order'
    CANCEL_ORDER = 'cancel_order'
    
    QUERY_ACCOUNT = 'query_account'
    QUERY_UNIVERSE = 'query_universe'
    QUERY_PORTFOLIO = 'query_portfolio'
    QUERY_POSITION = 'query_position'
    QUERY_ORDER = 'query_order'
    QUERY_TASK = 'query_task'
    QUERY_TRADE = 'query_trade'
    
    GOAL_PORTFOLIO = 'goal_portfolio'
    STOP_PORTFOLIO = 'stop_portfolio'
    
    TRADE_API_DISCONNECTED = 'trade_api_disconnected'
    TRADE_API_CONNECTED = 'trade_api_connected'



# encoding: UTF-8
from enum import Enum, unique


class ReprEnum(Enum):
    def __repr__(self):
        return "{0:s}_{1:s}".format(self.__class__.__name__,
                                    self._name_)
    
    def __str__(self):
        return "{0:s}_{1:s}".format(self.__class__.__name__,
                                    self._name_)
    
    @property
    def full_name(self):
        return str(self)
    
    @classmethod
    def to_enum(cls, key):
        return cls.__members__.get(key.upper(), None)


class ReprIntEnum(int, ReprEnum):
    """Enum where members are also (and must be) ints"""
    pass


class ReprStrEnum(str, ReprEnum):
    """Enum where members are also (and must be) ints"""
    pass


@unique
class QUOTE_TYPE(ReprStrEnum):
    TICK = '0'
    MIN = '1M'
    FIVEMIN = '5M'
    QUARTERMIN = '15M'
    DAILY = '1d'
    SPECIALBAR = '-1'
    # %Y%m%d%H%M%S


@unique
class RUN_MODE(ReprIntEnum):
    REALTIME = 0
    BACKTEST = 1


@unique
class EXCHANGE(ReprStrEnum):
    SHENZHEN_STOCK_EXCHANGE = 'SZ'
    SHANGHAI_STOCK_EXCHANGE = 'SH'
    
    SHANGHAI_FUTURES_EXCHANGE = 'SHF'
    ZHENGZHOU_COMMODITIES_EXCHANGE = 'CZC'
    DALIAN_COMMODITIES_EXCHANGE = 'DCE'
    
    CHINA_FINANCIAL_FUTURES_EXCHANGE = 'CFE'
    
    SHANGHAI_GOLD_EXCHANGE = 'SGE'
    
    CHINA_SECURITY_INDEX = 'CSI'
    
    HONGKONG_EXCHANGES_AND_CLEARING_LIMITED = 'HK'


@unique
class ORDER_TYPE(ReprStrEnum):
    """We recommend use limit order everywhere."""
    MARKET = "market"
    LIMIT = "limit"
    STOP = "stop"  # convert to market order once symbol price meet certain conditions.
    VWAP = "vwap"
    


@unique
class ORDER_ACTION(ReprStrEnum):
    """
    buy and sell for long; short and cover for short.
    other 4 actions are automatically generated by EMS."""
    BUY = "Buy"
    SELL = "Sell"
    SHORT = "Short"
    COVER = "Cover"
    SELLTODAY = "SellToday"
    SELLYESTERDAY = "SellYesterday"
    COVERYESTERDAY = "CoverYesterday"
    COVERTODAY = "CoverToday"
    
    @classmethod
    def is_positive(cls, action):
        return (action == cls.BUY
                or action == cls.COVER
                or action == cls.COVERYESTERDAY
                or action == cls.COVERTODAY)

    @classmethod
    def is_negative(cls, action):
        return (action == cls.SELL
                or action == cls.SHORT
                or action == cls.SELLTODAY
                or action == cls.SELLYESTERDAY)


@unique
class ORDER_STATUS(ReprStrEnum):
    NEW = "New"
    REJECTED = "Rejected"
    ACCEPTED = "Accepted"
    FILLED = "Filled"
    CANCELLED = "Cancelled"


@unique
class TASK_STATUS(ReprStrEnum):
    NEW = "New"
    REJECTED = "Rejected"
    ACCEPTED = "Accepted"
    DONE = "Done"
    CANCELLED = "Cancelled"


@unique
class ORDER_TIME_IN_FORCE(ReprStrEnum):
    FOK = 'fok'
    FAK = 'fak'
    IOC = 'ioc'


@unique
class CALENDAR_CONST(ReprIntEnum):
    TRADE_DAYS_PER_YEAR = 242


if __name__ == "__main__":
    """What below are actually unit tests. """
    
    print "Running test..."
    
    assert QUOTE_TYPE.TICK == '0'
    assert RUN_MODE.BACKTEST == 1
    assert ORDER_ACTION.BUY == 'Buy'
    assert ORDER_TYPE.MARKET == 'market'
    assert ORDER_STATUS.FILLED == 'Filled'
    
    print "Test passed."



# encoding: UTF-8

import copy
from collections import defaultdict

from jaqs.data.basic import OrderStatusInd, Trade, Task, Order, Position, TradeStat
from jaqs.trade import common


class PortfolioManager(object):
    """
    Used to store relevant context of the strategy.

    Attributes
    ----------
    orders : list of jaqs.data.basic.Order objects
    trades : list of jaqs.data.basic.Trade objects
    positions : dict of {symbol + trade_date : jaqs.data.basic.Position}
    strategy : Strategy
    holding_securities : set of securities

    Notes
    -------
    Position is determined only by TradeInd
    TradeStat is generated from order/goal_portfolio, updated by TradeInd
    
    tasks will generated by add_task, updated by task_rsp and task_ind
    trades will only be stored.
    

    """
    
    # TODO want / frozen update
    def __init__(self):
        self.ctx = None
        
        self.orders = dict()
        self.tasks = dict()
        self.trades = []
        
        self.positions = dict()
        self.tradestat = dict()
        
        self.holding_securities = set()
    
    def init_from_config(self, props):
        self._hook_strategy()
        self.init_positions()
        
    def _hook_strategy(self):
        self.original_on_order_status = self.ctx.strategy.on_order_status
        self.ctx.strategy.on_order_status = self._on_order_status
        
        self.original_on_trade = self.ctx.strategy.on_trade
        self.ctx.strategy.on_trade = self._on_trade
        
        self.original_on_order_rsp = self.ctx.strategy.on_order_rsp
        self.ctx.strategy.on_order_rsp = self._on_order_rsp
        
        self.original_on_task_rsp = self.ctx.strategy.on_task_rsp
        self.ctx.strategy.on_task_rsp = self._on_task_rsp
    
    @staticmethod
    def _make_position_key(symbol):
        # return '@'.join(symbol)
        return symbol
    
    @staticmethod
    def _make_order_key(entrust_id, trade_date):
        return '@'.join((str(entrust_id), str(trade_date)))

    def _on_order_rsp(self, rsp):
        self.original_on_order_rsp(rsp)

    def _make_trade_stat_key(self, symbol):
        # return '@'.join(symbol)
        return '{:s}@{}'.format(symbol, self.ctx.trade_date)
    
    def _get_trade_stat(self, symbol):
        """
        
        Parameters
        ----------
        symbol : str

        Returns
        -------
        TradeStat

        """
        key = self._make_trade_stat_key(symbol)
        trade_stat = self.tradestat.get(key, None)
        if trade_stat is None:
            trade_stat = TradeStat(symbol=symbol)
        return trade_stat

    def get_trade_stat(self, symbol):
        """
        
        Parameters
        ----------
        symbol : str

        Returns
        -------
        TradeStat

        """
        key = self._make_trade_stat_key(symbol)
        trade_stat = self.tradestat.get(key, None)
        return trade_stat
    
    def get_position(self, symbol):
        pos_key = self._make_position_key(symbol)
        position = self.positions.get(pos_key, None)
        return position
    
    def get_pos(self, symbol):
        pos_key = self._make_position_key(symbol)
        position = self.positions.get(pos_key, None)
        if position is None:
            return 0
        else:
            return position.current_size

    def get_task(self, task_id):
        """
        
        Parameters
        ----------
        task_id : int

        Returns
        -------
        Task

        """
        return self.tasks.get(task_id, None)

    def init_positions(self):
        query_account_res = self.ctx.trade_api.query_account()
        if query_account_res is None:
            return
        
        df_acc, msg = self.ctx.trade_api.query_account()
        if msg != '0,':
            print(msg)
            raise RuntimeError("Query account failed")
        account_info = df_acc.set_index('type').to_dict(orient='index')
        
        df_univ, msg = self.ctx.trade_api.query_universe()
        df_univ = df_univ.rename(columns={'security': 'symbol'})
        univ = df_univ['symbol'].values.copy()
        # self.ctx.init_universe(univ)
        
        df_pos, msg = self.ctx.trade_api.query_position()
        df_pos = df_pos.rename(columns={'security': 'symbol'})
        pos_list = Position.create_from_df(df_pos)
        pos_dic = {p.symbol: p for p in pos_list}
        self.positions.update(pos_dic)
        
    # ----------------------------------------------------------------------------
    # On Task Change
    
    def add_task(self, task):
        """
        Add order to orders, create position and tradestat if necessary.

        Parameters
        ----------
        task : Task

        """
        if task.task_id in self.tasks:
            print 'duplicate task {}'.format(task.task_id)
        
        # Store Task (right after strategy constructs a task)
        # TODO: copy
        self.tasks[task.task_id] = copy.deepcopy(task)
        
        # Store/Update TradeStat (right after strategy constructs a task)
        if task.function_name == 'place_order':
            order = task.data
            # self.orders[order.entrust_no] = order
            self._update_trade_stat_from_order(order)
        
        elif task.function_name == 'place_batch_order':
            orders = task.data
            for order in orders:
                # self.orders[order.entrust_no] = order
                self._update_trade_stat_from_order(order)
        
        elif task.function_name == 'basket_order':
            # TODO: no Order class for basket_order
            raise NotImplementedError("basket_order")
        
        elif task.function_name == 'goal_portfolio':
            # self._update_trade_stat_from_goal_positions(goal_positions)
            orders = task.data
            for order in orders:
                self._update_trade_stat_from_order(order)
    
    def _on_task_rsp(self, rsp):
        """
        
        Parameters
        ----------
        rsp : TaskRsp

        """
        task_id = rsp.task_id
        task = self.tasks.get(task_id, None)
        if task is None:
            print("task_id {} does not exist in PortfolioManager".format(task_id))
            return
        
        if rsp.success:
            task.task_id = rsp.task_id
            
            # write back
            self.tasks[task_id] = task
        else:
            # Update TradeStat (right after strategy constructs a task)
            if task.function_name == 'place_order':
                order = task.data
                self._update_trade_stat_from_order(order, roll_back=True)
            
            elif task.function_name == 'place_batch_order':
                orders = task.data
                for order in orders:
                    self._update_trade_stat_from_order(order, roll_back=True)
            
            elif task.function_name == 'basket_order':
                # TODO: no Order class for basket_order
                raise NotImplementedError("basket_order")
            
            elif task.function_name == 'goal_portfolio':
                goal_positions = task.data
                self._update_trade_stat_from_goal_positions(goal_positions, roll_back=True)
    
    def _update_trade_stat_from_order(self, order, roll_back=False):
        """
        
        Parameters
        ----------
        order : Order
            Not yet filled.
        roll_back : bool
            if the order is canceled, or task is failed/stopped, we roll back our previous change.

        """
        sign = 1
        if roll_back:
            sign *= -1
        
        tradestat = self._get_trade_stat(order.symbol)
        
        # if order.entrust_action == common.ORDER_ACTION.BUY:
        if common.ORDER_ACTION.is_positive(order.entrust_action):
            tradestat.buy_want_size += order.entrust_size * sign
        else:
            tradestat.sell_want_size += order.entrust_size * sign
        
        self.tradestat[self._make_trade_stat_key(order.symbol)] = tradestat
    
    def _update_trade_stat_from_goal_positions(self, positions, roll_back=False):
        """
        
        Parameters
        ----------
        positions : list of dict
            [ {"security": "000001.SZ", "ref_price": 10.0, "size" : 100}, ...]
        roll_back : bool
            if the order is canceled, or task is failed/stopped, we roll back our previous change.

        """
        sign = 1
        if roll_back:
            sign *= -1
        
        for goal_pos in positions:
            size = goal_pos['size']
            if size == 0:
                continue
            symbol = goal_pos['symbol']
            
            tradestat = self._get_trade_stat(symbol)
            
            # tradestat.buy_want_size = 0
            # tradestat.sell_want_size = 0
            if size > 0:
                tradestat.buy_want_size += size * sign
            else:
                tradestat.sell_want_size += size * sign
            
            self.tradestat[self._make_trade_stat_key(symbol)] = tradestat
    
    '''
    def add_order(self, order):
        """
        Add order to orders, create position and tradestat if necessary.

        Parameters
        ----------
        order : Order

        """
        if order.entrust_no in self.orders:
            print 'duplicate entrust_no {}'.format(order.entrust_no)
            return False
        
        new_order = Order()
        new_order.copy(order)  # TODO why copy?
        self.orders[self._make_order_key(order.entrust_no, self.ctx.strategy.ctx.trade_date)] = new_order
        
        position_key = self._make_position_key(order.symbol)
        if position_key not in self.positions:
            position = Position()
            position.symbol = order.symbol
            self.positions[position_key] = position
        
        if order.symbol not in self.tradestat:
            tradestat = TradeStat()
            tradestat.symbol = order.symbol
            self.tradestat[order.symbol] = tradestat
        
        tradestat = self.tradestat.get(order.symbol)
        
        if order.entrust_action == common.ORDER_ACTION.BUY:
            tradestat.buy_want_size += order.entrust_size
        else:
            tradestat.sell_want_size += order.entrust_size

    '''
    
    # ----------------------------------------------------------------------------
    # On Order Status
    
    def _update_task_if_done(self, task_id):
        task = self.get_task(task_id)
        if task.function_name == 'place_order':
            order = task.data
            if order.is_finished:
                task.task_status = common.TASK_STATUS.DONE
        elif task.function_name == 'place_batch_order':
            orders = task.data
            if all([o.is_finished for o in orders]):
                task.task_status = common.TASK_STATUS.DONE
        elif task.function_name == 'goal_portfolio':
            orders = task.data
            if all([o.is_finished for o in orders]):
                task.task_status = common.TASK_STATUS.DONE
            '''
            goals = task.data
            if all([((self.get_position(goal['symbol']) is not None)
                     and (self.get_position(goal['symbol']).current_size == goal['size']))
                    for goal in goals]):
                task.task_status = common.TASK_STATUS.DONE
            '''
        else:
            raise NotImplementedError()
        
        self.tasks[task_id] = task
        
    def _on_order_status(self, ind):
        """
        
        Parameters
        ----------
        ind : OrderStatusInd

        """
        
        if ind.task_id not in self.tasks:
            return

        # TODO
        if ind.entrust_no == 101010 or ind.entrust_no == 202020:  # trades generate by system
            return

        # add/update order
        order = self.orders.get(ind.entrust_no, None)
        if order is None:
            order = Order()
        order.copy(ind)
        self.orders[ind.entrust_no] = order
        
        task = self.get_task(ind.task_id)
        if task.function_name == 'place_order':
            order = task.data
            order.copy(ind)
        elif task.function_name == 'place_batch_order':
            for order in task.data:
                if order.entrust_no == ind.entrust_no:
                    order.copy(ind)
        elif task.function_name == 'goal_portfolio':
            for order in task.data:
                if order.entrust_no == ind.entrust_no:
                    order.copy(ind)
        
        # order status other than CANCELLED/REJECTED will be dealt with self.on_trade
        if (ind.order_status == common.ORDER_STATUS.CANCELLED) or (ind.order_status == common.ORDER_STATUS.REJECTED):
            
            # update TradeStat
            trade_stat = self._get_trade_stat(ind.symbol)
            
            release_size = ind.entrust_size - ind.fill_size
            if common.ORDER_ACTION.is_positive(ind.entrust_action):
                trade_stat.buy_want_size -= release_size
            else:
                trade_stat.sell_want_size -= release_size
            
            self.tradestat[self._make_trade_stat_key(ind.symbol)] = trade_stat

            # update task
            self._update_task_if_done(ind.task_id)
            
            """
            entrust_no = ind.entrust_no
            order = self.orders.get(self._make_order_key(entrust_no, self.ctx.strategy.ctx.trade_date), None)
            if order is not None:
                order.order_status = ind.order_status
            
                tradestat = self.tradestat.get(ind.symbol)
                release_size = ind.entrust_size - ind.fill_size
            
                if ind.entrust_action == common.ORDER_ACTION.BUY:
                    tradestat.buy_want_size -= release_size
                else:
                    tradestat.sell_want_size -= release_size
            else:
                raise ValueError("order {} does not exist".format(entrust_no))
            """
        self.original_on_order_status(ind)
    
    '''
    def on_order_status_OLD(self, ind):
        if ind.order_status is None:
            return
        
        if (ind.order_status == common.ORDER_STATUS.CANCELLED
            or ind.order_status == common.ORDER_STATUS.REJECTED):
            
            entrust_no = ind.entrust_no
            order = self.orders.get(self._make_order_key(entrust_no, self.ctx.strategy.ctx.trade_date), None)
            if order is not None:
                order.order_status = ind.order_status
                
                tradestat = self.tradestat.get(ind.symbol)
                release_size = ind.entrust_size - ind.fill_size
                
                if ind.entrust_action == common.ORDER_ACTION.BUY:
                    tradestat.buy_want_size -= release_size
                else:
                    tradestat.sell_want_size -= release_size
            else:
                raise ValueError("order {} does not exist".format(entrust_no))

        self.original_on_order_status(ind)
    
    '''
    '''
    def set_position(self, symbol, date, ratio=1):
        """Modify latest (thus date might not be necessary) position by a ratio."""
        pos_key = self._make_position_key(symbol)
        pos = self.positions.get(pos_key)
        
        pos.current_size *= ratio
        pos.init_size *= ratio
        self.positions[pos_key] = pos
    
    '''
    
    # ----------------------------------------------------------------------------
    # On Trade Indication
    
    def _on_trade(self, ind):
        # record trades
        self.trades.append(ind)
        
        '''
        # Change Order Status
        entrust_no = ind.entrust_no
        if entrust_no == 101010 or 202020:  # trades generate by system
            pass
        else:
            order = self.orders.get(self._make_order_key(entrust_no, self.ctx.strategy.ctx.trade_date), None)
            if order is None:
                print 'cannot find order for entrust_no' + entrust_no
                return
            
            order.fill_size += ind.fill_size
            
            if order.fill_size == order.entrust_size:
                order.order_status = common.ORDER_STATUS.FILLED
            else:
                order.order_status = common.ORDER_STATUS.ACCEPTED
        '''
        
        # Change Position
        self._update_position_by_trade_ind(ind)
        
        # Change TradeStat
        self._update_trade_stat_from_trade_ind(ind)

        # Update Orders
        # self._update_order_from_trade_ind(ind)

        # Update Tasks
        if not (ind.entrust_no == 101010 or ind.entrust_no == 202020):  # trades generate by system
            self._update_task_if_done(ind.task_id)
        
        # hook:
        self.original_on_trade(ind)
    
    def _update_order_from_trade_ind(self, ind):
        order = self.orders.get(ind.entrust_no)
        if order is None:
            return
        
        order.fill_size += ind.fill_size

        task = self.get_task(ind.task_id)
        if task.function_name == 'place_order':
            order = task.data
            order.fill_size += ind.fill_size
        elif task.function_name == 'place_batch_order':
            for order in task.data:
                order.fill_size += ind.fill_size
        else:
            raise NotImplementedError()
    
    def _update_trade_stat_from_trade_ind(self, ind):
        """
        
        Parameters
        ----------
        ind : Trade
            Not yet filled.

        """
        tradestat = self._get_trade_stat(ind.symbol)
        
        if common.ORDER_ACTION.is_positive(ind.entrust_action):
            tradestat.buy_filled_size += ind.fill_size
            tradestat.buy_want_size -= ind.fill_size
        elif common.ORDER_ACTION.is_negative(ind.entrust_action):
            tradestat.sell_filled_size += ind.fill_size
            tradestat.sell_want_size -= ind.fill_size
        
        self.tradestat[self._make_trade_stat_key(ind.symbol)] = tradestat
    
    def _update_position_by_trade_ind(self, ind):
        # ignore no fill_size (which should not be)
        if ind.fill_size == 0:
            print("WARNING: no fill_size TradeInd found!")
            return
        
        # get position, if no, create a new one.
        pos_key = self._make_position_key(ind.symbol)
        pos = self.positions.get(pos_key, None)
        if pos is None:
            pos = Position(symbol=ind.symbol)
        
        if common.ORDER_ACTION.is_positive(ind.entrust_action):
            pos.current_size += ind.fill_size
        elif common.ORDER_ACTION.is_negative(ind.entrust_action):
            pos.current_size -= ind.fill_size
        
        self.positions[pos_key] = pos
        
        # if no holding, remove the position from the dict
        if pos.current_size == 0:
            self.positions.pop(pos_key)
            # TODO : remove holding_securities field
            self.holding_securities.remove(ind.symbol)
        else:
            self.holding_securities.add(ind.symbol)
    
    # ----------------------------------------------------------------------------
    # For Alpha Strategy
    
    def market_value(self, ref_prices, suspensions=None):
        """
        Calculate total market value according to all current positions.
        NOTE for now this func only support stocks.

        Parameters
        ----------
        ref_prices : dict of {symbol: price}
            The prices we refer to to get symbol price.
        suspensions : list of securities
            Securities that are suspended.

        Returns
        -------
        market_value : float

        """
        # TODO some securities could not be able to be traded
        if suspensions is None:
            suspensions = []
        
        market_value_float = 0.0
        market_value_frozen = 0.0  # suspended or high/low limit
        for sec in self.holding_securities:
            size = self.get_position(sec).current_size
            # TODO PortfolioManager object should not access price
            price = ref_prices[sec]
            mv_sec = price * size
            if sec in suspensions:
                market_value_frozen += mv_sec
            else:
                market_value_float += mv_sec
        
        return market_value_float, market_value_frozen


'''
class PortfolioManager_RAW(TradeCallback):
    """
    Used to store relevant context of the strategy.

    Attributes
    ----------
    orders : list of jaqs.data.basic.Order objects
    trades : list of jaqs.data.basic.Trade objects
    positions : dict of {symbol + trade_date : jaqs.data.basic.Position}
    strategy : Strategy
    holding_securities : set of securities

    Methods
    -------

    """
    
    # TODO want / frozen update
    def __init__(self, strategy=None):
        self.orders = {}
        self.trades = []
        self.positions = {}
        self.holding_securities = set()
        self.tradestat = {}
        self.ctx.strategy = strategy
    
    @staticmethod
    def _make_position_key(symbol, trade_date):
        return '@'.join((symbol, str(trade_date)))

    @staticmethod
    def _make_order_key(entrust_id, trade_date):
        return '@'.join((str(entrust_id), str(trade_date)))
    
    def on_order_rsp(self, order, result, msg):
        if result:
            self.add_order(order)
    
    def get_position(self, symbol, date):
        key = self._make_position_key(symbol, date)
        position = self.positions.get(key, None)
        return position
    
    def on_new_day(self, date, pre_date):
        for key, pos in self.positions.viewitems():
            sec, td = key.split('@')
            if str(pre_date) == td:
                new_key = self._make_position_key(sec, date)
                pre_position = pos
                
                new_position = Position()
                new_position.current_size = pre_position.current_size
                new_position.init_size = new_position.current_size
                new_position.symbol = pre_position.symbol
                new_position.trade_date = date
                self.positions[new_key] = new_position
        
        """
        for sec in self.holding_securities:
            pre_key = self._make_position_key(sec, pre_date)
            new_key = self._make_position_key(sec, date)
            if pre_key in self.positions:
                pre_position = self.positions.get(pre_key)
                new_position = Position()
                new_position.current_size = pre_position.current_size
                new_position.init_size = new_position.current_size
                new_position.symbol = pre_position.symbol
                new_position.trade_date = date
                self.positions[new_key] = new_position
        """
    
    def add_order(self, order):
        """
        Add order to orders, create position and tradestat if necessary.

        Parameters
        ----------
        order : Order

        """
        if order.entrust_no in self.orders:
            print 'duplicate entrust_no {}'.format(order.entrust_no)
            return False
        
        new_order = Order()
        new_order.copy(order)  # TODO why copy?
        self.orders[self._make_order_key(order.entrust_no, self.ctx.strategy.ctx.trade_date)] = new_order
        
        position_key = self._make_position_key(order.symbol, self.ctx.strategy.ctx.trade_date)
        if position_key not in self.positions:
            position = Position()
            position.symbol = order.symbol
            self.positions[position_key] = position
        
        if order.symbol not in self.tradestat:
            tradestat = TradeStat()
            tradestat.symbol = order.symbol
            self.tradestat[order.symbol] = tradestat
        
        tradestat = self.tradestat.get(order.symbol)
        
        if order.entrust_action == common.ORDER_ACTION.BUY:
            tradestat.buy_want_size += order.entrust_size
        else:
            tradestat.sell_want_size += order.entrust_size
    
    def on_order_status(self, ind):
        if ind.order_status is None:
            return
        
        if ind.order_status == common.ORDER_STATUS.CANCELLED or ind.order_status == common.ORDER_STATUS.REJECTED:
            entrust_no = ind.entrust_no
            order = self.orders.get(self._make_order_key(entrust_no, self.ctx.strategy.ctx.trade_date), None)
            if order is not None:
                order.order_status = ind.order_status
                
                tradestat = self.tradestat.get(ind.symbol)
                release_size = ind.entrust_size - ind.fill_size
                
                if ind.entrust_action == common.ORDER_ACTION.BUY:
                    tradestat.buy_want_size -= release_size
                else:
                    tradestat.sell_want_size -= release_size
            else:
                raise ValueError("order {} does not exist".format(entrust_no))
    
    def set_position(self, symbol, date, ratio=1):
        """Modify latest (thus date might not be necessary) position by a ratio."""
        pos_key = self._make_position_key(symbol, date)
        pos = self.positions.get(pos_key)

        pos.current_size *= ratio
        pos.init_size *= ratio
        self.positions[pos_key] = pos
        
    def on_trade(self, ind):
        entrust_no = ind.entrust_no
        
        order = self.orders.get(self._make_order_key(entrust_no, self.ctx.strategy.ctx.trade_date), None)
        if order is None:
            print 'cannot find order for entrust_no' + entrust_no
            return
        
        self.trades.append(ind)
        
        order.fill_size += ind.fill_size
        
        if order.fill_size == order.entrust_size:
            order.order_status = common.ORDER_STATUS.FILLED
        else:
            order.order_status = common.ORDER_STATUS.ACCEPTED
        
        position_key = self._make_position_key(ind.symbol, self.ctx.strategy.ctx.trade_date)
        position = self.positions.get(position_key)
        tradestat = self.tradestat.get(ind.symbol)
        
        if (ind.entrust_action == common.ORDER_ACTION.BUY
            or ind.entrust_action == common.ORDER_ACTION.COVER
            or ind.entrust_action == common.ORDER_ACTION.COVERYESTERDAY
            or ind.entrust_action == common.ORDER_ACTION.COVERTODAY):
            
            tradestat.buy_filled_size += ind.fill_size
            tradestat.buy_want_size -= ind.fill_size
            
            position.current_size += ind.fill_size
        
        elif (ind.entrust_action == common.ORDER_ACTION.SELL
              or ind.entrust_action == common.ORDER_ACTION.SELLTODAY
              or ind.entrust_action == common.ORDER_ACTION.SELLYESTERDAY
              or ind.entrust_action == common.ORDER_ACTION.SHORT):
            
            tradestat.sell_filled_size += ind.fill_size
            tradestat.sell_want_size -= ind.fill_size
            
            position.current_size -= ind.fill_size
        
        if position.current_size != 0:
            self.holding_securities.add(ind.symbol)
        else:
            self.holding_securities.remove(ind.symbol)
    
    def market_value(self, ref_date, ref_prices, suspensions=None):
        """
        Calculate total market value according to all current positions.
        NOTE for now this func only support stocks.

        Parameters
        ----------
        ref_date : int
            The date we refer to to get symbol position.
        ref_prices : dict of {symbol: price}
            The prices we refer to to get symbol price.
        suspensions : list of securities
            Securities that are suspended.

        Returns
        -------
        market_value : float

        """
        # TODO some securities could not be able to be traded
        if suspensions is None:
            suspensions = []
        
        market_value = 0.0
        for sec in self.holding_securities:
            if sec in suspensions:
                continue
            
            size = self.get_position(sec, ref_date).current_size
            # TODO PortfolioManager object should not access price
            price = ref_prices[sec]
            market_value += price * size * 100
        
        return market_value


'''




from setuptools import setup, find_packages
from jaqs import __version__ as ver
import codecs
import os


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname), 'r', encoding='utf-8').read()


def readme():
    with codecs.open('README.rst', 'r', encoding='utf-8') as f:
        return f.read()


def read_install_requires():
    with codecs.open('requirements.txt', 'r', encoding='utf-8') as f:
        res = f.readlines()
    res = list(map(lambda s: s.replace('\n', ''), res))
    return res

setup(
    name='jaqs',
    version=ver,
    description='Open source quantitative research&trading framework.',
    long_description=readme(),
    install_requires=read_install_requires(),
    license='Apache 2.0',
    classifiers=[
        'Programming Language :: Python :: 2.7',
    ],
    packages=find_packages(),
    package_data={'': ['*.json', '*.css', '*.html']},
)



# encoding: utf-8

import numpy as np
import pandas as pd
import scipy.stats as scst
import statsmodels.api as sm

from jaqs.trade.common import CALENDAR_CONST


def calc_signal_ic(signal_data):
    """
    Computes the Spearman Rank Correlation based Information Coefficient (IC)
    between signal values and N period forward returns for each period in
    the signal index.

    Parameters
    ----------
    signal_data : pd.DataFrame - MultiIndex
        Index is pd.MultiIndex ['trade_date', 'symbol'], columns = ['signal', 'return', 'quantile']

    Returns
    -------
    ic : pd.DataFrame
        Spearman Rank correlation between signal and provided forward returns.
        
    """
    def src_ic(df):
        _ic = scst.spearmanr(df['signal'], df['return'])[0]
        return _ic

    signal_data = signal_data.copy()

    grouper = ['trade_date']

    ic = signal_data.groupby(grouper).apply(src_ic)
    ic = pd.DataFrame(ic)
    ic.columns = ['ic']

    return ic


def calc_ic_stats_table(ic_data):
    ic_summary_table = pd.DataFrame()
    ic_summary_table["IC Mean"] = ic_data.mean()
    ic_summary_table["IC Std."] = ic_data.std()
    t_stat, p_value = scst.ttest_1samp(ic_data, 0)
    ic_summary_table["t-stat(IC)"] = t_stat
    ic_summary_table["p-value(IC)"] = p_value
    ic_summary_table["IC Skew"] = scst.skew(ic_data)
    ic_summary_table["IC Kurtosis"] = scst.kurtosis(ic_data)
    ic_summary_table["Ann. IR"] = ic_data.mean() / ic_data.std()


def mean_information_coefficient(ic, by_time=None):
    """
    Get the mean information coefficient of specified groups.
    Answers questions like:
    What is the mean IC for each month?
    What is the mean IC for each group for our whole timerange?
    What is the mean IC for for each group, each week?

    Parameters
    ----------
    by_time : str (pd time_rule), optional
        Time window to use when taking mean IC.
        See http://pandas.pydata.org/pandas-docs/stable/timeseries.html
        for available options.

    Returns
    -------
    ic : pd.DataFrame
        Mean Spearman Rank correlation between signal and provided
        forward price movement windows.
    """

    grouper = []
    if by_time is not None:
        grouper.append(pd.TimeGrouper(by_time))

    if len(grouper) == 0:
        ic = ic.mean()

    else:
        ic.index = pd.to_datetime(ic.index, format="%Y%m%d")
        ic = (ic.reset_index().set_index('trade_date').groupby(grouper).mean())

    return ic


def calc_period_wise_weighted_signal_return(signal_data, weight_method):
    """
    Computes period wise period_wise_returns for portfolio weighted by signal
    values. Weights are computed by demeaning signals and dividing
    by the sum of their absolute value (achieving gross leverage of 1).

    Parameters
    ----------
    signal_data : pd.DataFrame - MultiIndex
        Index is pd.MultiIndex ['trade_date', 'symbol'], columns = ['signal', 'return', 'quantile']
    weight_method : {'equal_weight', 'long_only', 'long_short'}

    Returns
    -------
    res : pd.DataFrame
        Period wise period_wise_returns of dollar neutral portfolio weighted by signal value.
        
    """
    def calc_norm_weights(ser, method):
        if method == 'equal_weight':
            ser.loc[:] = 1.0 / len(ser)
        elif method == 'long_short':
            # TODO: do we need to de-mean?
            ser = ser - ser.mean()
        elif method == 'long_only':
            ser = (ser + ser.abs()) / 2.0
        elif method == 'short_only':
            ser = (ser - ser.abs()) / 2.0
        else:
            raise ValueError("method can only be equal_weight, long_only or long_short,"
                             "but [{}] is provided".format(method))
        return ser / ser.abs().sum()
    
    grouper = ['trade_date']
    
    weights = signal_data.groupby(grouper)['signal'].apply(calc_norm_weights, weight_method)
    # df_sig = signal_data['signal'].unstack(level='symbol')
    # weights = df_sig.apply(calc_norm_weights, axis=1, args=(weight_method, ))
    
    weighted_returns = signal_data['return'].multiply(weights, axis=0)
    
    period_wise_returns = weighted_returns.groupby(level='trade_date').sum()
    
    res = pd.DataFrame(period_wise_returns)
    res.columns = ['return']
    return res


def regress_period_wise_signal_return(signal_data, group=False):
    """
    Computes period wise period_wise_returns for portfolio weighted by signal
    values. Weights are computed by demeaning signals and dividing
    by the sum of their absolute value (achieving gross leverage of 1).

    Parameters
    ----------
    signal_data : pd.DataFrame - MultiIndex
        Index is pd.MultiIndex ['trade_date', 'symbol'], columns = ['signal', 'return', 'quantile']

    Returns
    -------
    period_wise_returns : pd.DataFrame
        Period wise period_wise_returns of dollar neutral portfolio weighted by signal
        value.
    """
    
    def regress(df):
        x = df['signal'].values
        y = df['return'].values
        x = sm.add_constant(x)

        mod = sm.OLS(y, x).fit()
        idiosyncractic, signal_return = mod.params
        # return pd.Series(index=['idio', 'signal_return'], data=[idiosyncractic, signal_return])
        return signal_return
    
    grouper = [signal_data.index.get_level_values('trade_date')]
    if group:
        grouper.append('group')
    
    regress_res = signal_data.groupby(grouper).apply(regress)
    
    return pd.DataFrame(regress_res)


'''
def calc_alpha_beta(active_return, period, benchmark_return=None):
    if isinstance(active_return, pd.Series):
        active_return = pd.DataFrame(active_return)
    if isinstance(benchmark_return, pd.Series):
        benchmark_return = pd.DataFrame(benchmark_return)
    benchmark_return = benchmark_return.loc[active_return.index, :]

    alpha_beta = pd.DataFrame()
    
    x = benchmark_return.values
    y = active_return.values
    x = sm.add_constant(x)

    reg_fit = sm.OLS(y, x).fit()
    alpha, beta = reg_fit.params

    alpha_beta.loc['Ann. alpha', period] = \
        (1 + alpha) ** (252.0 / period) - 1
    alpha_beta.loc['beta', period] = beta

    return alpha_beta


'''


def calc_quantile_return_mean_std(signal_data, time_series=False):
    """
    Computes mean returns for signal quantiles across
    provided forward returns columns.

    Parameters
    ----------
    signal_data : pd.DataFrame - MultiIndex
        Index is pd.MultiIndex ['trade_date', 'symbol'], columns = ['signal', 'return', 'quantile']

    Returns
    -------
    res : pd.DataFrame of dict
    
    """
    signal_data = signal_data.copy()
    
    grouper = ['quantile']
    if time_series:
        grouper.append('trade_date')
    
    group_mean_std = signal_data.groupby(grouper)['return'].agg(['mean', 'std', 'count'])

    # TODO: why?
    '''
    std_error_ret = group_mean_std.loc[:, 'std'].copy() / np.sqrt(group_mean_std.loc[:, 'count'].copy())
    '''

    if time_series:
        quantile_daily_mean_std_dic = dict()
        quantiles = np.unique(group_mean_std.index.get_level_values(level='quantile'))
        for q in quantiles:  # loop for different quantiles
            df_q = group_mean_std.loc[pd.IndexSlice[q, :], :]  # bug
            df_q.index = df_q.index.droplevel(level='quantile')
            quantile_daily_mean_std_dic[q] = df_q
        return quantile_daily_mean_std_dic
    else:
        return group_mean_std


def calc_return_diff_mean_std(q1, q2):
    """
    Computes the difference between the mean returns of
    two quantiles. Optionally, computes the standard error
    of this difference.

    Parameters
    ----------
    q1, q2 : pd.DataFrame
        DataFrame of mean period wise returns by quantile.
        Index is datet, columns = ['mean', 'std', 'count']

    Returns
    -------
    res : pd.DataFrame
        Difference of mean return and corresponding std.
        
    """
    assert np.all(q1.index == q2.index)
    assert np.all(q1.columns == q2.columns)
    
    res = pd.DataFrame(index=q1.index, columns=['mean_diff', 'std'])
    res.loc[:, 'mean_diff'] = q1['mean'] - q2['mean']
    res.loc[:, 'std'] = np.sqrt(q1['std']**2 + q2['std']**2)
    return res


'''
def period2daily(ser, period, do_roll_mean=False):
    if not period > 1:
        return ser
    
    if do_roll_mean:
        ser = ser.rolling(window=period, min_periods=1, axis=1).mean()
    
    ser_daily_pow = (ser + 1) ** (1. / period)
    return ser_daily_pow - 1.0
'''


def calc_active_cum_return_way2(portfolio_ret, benchmark_ret):
    benchmark_ret = benchmark_ret.loc[portfolio_ret.index]
    
    portfolio_cum = portfolio_ret.add(1.0).cumprod(axis=0)
    benchmark_cum = benchmark_ret.add(1.0).cumprod(axis=0)
    active_cum = portfolio_cum.sub(benchmark_cum.values.flatten(), axis=0) + 1.0
    return active_cum


def calc_active_cum_return(portfolio_ret, benchmark_ret):
    benchmark_ret = benchmark_ret.loc[portfolio_ret.index]
    
    active_ret = portfolio_ret.sub(benchmark_ret.values.flatten(), axis=0)
    active_cum = active_ret.add(1.0).cumprod()
    return active_cum


def daily_ret_to_cum2(df_ret, axis=0):
    cum = df_ret.add(1.0).cumprod(axis=axis)
    return cum


def price2ret(prices, period=5, axis=None):
    """

    Parameters
    ----------
    prices : pd.DataFrame or pd.Series
        Index is datetime.
    period : int
    axis : {0, 1, None}
    
    Returns
    -------
    ret : pd.DataFrame or pd.Series
    
    """
    ret = cum2ret(prices, period=period, axis=axis, compound=True)
    return ret


def cum2ret(cum, period=1, axis=None, compound=False):
    """
    
    Parameters
    ----------
    cum : pd.Series
        Starts from zero.
    period : int
    axis : {0, 1, None}
    compound : bool

    Returns
    -------
    ret : pd.Series

    """
    if axis is None:
        kwargs = dict()
    else:
        kwargs = {'axis': axis}
    
    if np.any(cum.min(**kwargs)) < 0:
        raise ValueError("Minimum value of cumulative return is less than zero.")
    cum = cum.add(1.0)
    if compound:
        ret = cum.pct_change(periods=period, **kwargs).fillna(0.0)
    else:
        ret = cum.diff(periods=period, **kwargs).fillna(0.0)
    return ret


def ret2cum(ret, compound=False, axis=None):
    """
    
    Parameters
    ----------
    ret : pd.Series
        Starts from zero.
    compound : bool
    axis : {0, 1, None}

    Returns
    -------
    cum : pd.Series

    """
    if axis is None:
        kwargs = dict()
    else:
        kwargs = {'axis': axis}
    if compound:
        # use log to avoid numerical problems
        log_sum = np.log(ret.add(1.0)).cumsum(**kwargs)
        cum = np.exp(log_sum).sub(1.0)
    else:
        cum = ret.cumsum(**kwargs)
    return cum


def calc_performance_metrics(ser, cum_return=False, compound=False):
    """
    Calculate annualized return, volatility and sharpe.
    We assumed data frequency to be day.

    Parameters
    ----------
    ser : pd.DataFrame or pd.Series
        Index is int date, values are floats.
        ser should start from 0.
    cum_return : bool
        Whether ser is cumulative or daily return.
    compound
        Whether calculation of return is compound.

    Returns
    -------
    res : dict

    """
    if isinstance(ser, pd.DataFrame):
        ser = ser.iloc[:, 0]
    
    idx = ser.index
    
    if cum_return:
        cum_ret = ser
        ret = cum2ret(cum_ret, period=1, compound=compound)
    else:
        ret = ser
        cum_ret = ret2cum(ret, compound=compound)
    
    n_trade_days = len(idx)
    n_years = n_trade_days * 1. / CALENDAR_CONST.TRADE_DAYS_PER_YEAR
    
    total_ret = cum_ret.iat[-1]
    if compound:
        ann_ret = np.power(cum_ret.iat[-1] + 1.0, 1. / n_years) - 1
    else:
        ann_ret = total_ret / n_years
    std = np.std(ret)  # use std instead of np.sqrt( (ret**2).sum() / len(ret) )
    ann_vol = std * np.sqrt(CALENDAR_CONST.TRADE_DAYS_PER_YEAR)
    sharpe = ann_ret / ann_vol
    # print "ann. ret = {:.1f}%; ann. vol = {:.1f}%, sharpe = {:.2f}".format(ann_ret * 100, ann_vol * 100, sharpe)
    res = {'ann_ret': ann_ret,
           'ann_vol': ann_vol,
           'sharpe': sharpe}
    return res


def period_wise_ret_to_cum(ret, period, compound=False):
    """
    Calculate cumulative returns from N-periods returns, no compounding.
    When 'period' N is greater than 1 the cumulative returns plot is computed
    building and averaging the cumulative returns of N interleaved portfolios
    (started at subsequent periods 1,2,3,...,N) each one rebalancing every N
    periods.
    
    Parameters
    ----------
    ret: pd.Series or pd.DataFrame
        pd.Series containing N-periods returns
    period: integer
        Period for which the returns are computed
    compound : bool
        Whether calculate using compound return.
    
    Returns
    -------
    pd.Series
        Cumulative returns series starting from zero.
        
    """
    if isinstance(ret, pd.DataFrame):
        # deal with each column recursively
        return ret.apply(period_wise_ret_to_cum, axis=0, args=(period,))
    elif isinstance(ret, pd.Series):
        if period == 1:
            return ret.add(1).cumprod().sub(1.0)
        
        # invest in each portfolio separately
        periods_index = np.arange(len(ret.index)) // period
        period_portfolios = ret.groupby(by=periods_index, axis=0).apply(lambda ser: pd.DataFrame(np.diag(ser)))
        period_portfolios.index = ret.index
        
        # cumulate returns separately
        if compound:
            cum_returns = period_portfolios.add(1).cumprod().sub(1.0)
        else:
            cum_returns = period_portfolios.cumsum()
        
        # since capital of all portfolios are the same, return in all equals average return
        res = cum_returns.mean(axis=1)
        
        return res
    else:
        raise NotImplementedError("ret must be Series or DataFrame.")

